#!/usr/bin/env python3

from functools import wraps
import scat.util as util
import logging

class DiagFallbackEventParser:
    def __init__(self, parent):
        self.parent = parent
        self.header = b''

        # Event IDs are available at:
        # https://source.codeaurora.org/quic/la/platform/vendor/qcom-opensource/wlan/qcacld-2.0/tree/CORE/VOSS/inc/event_defs.h
        # https://android.googlesource.com/kernel/msm/+/android-7.1.0_r0.2/drivers/staging/qcacld-2.0/CORE/VOSS/inc/event_defs.h
        self.event_names = {
            # event ID, event name
            256: 'EVENT_BAND_CLASS_CHANGE', # 0x100
            257: 'EVENT_CDMA_CH_CHANGE', # 0x101
            258: 'EVENT_BS_P_REV_CHANGE', # 0x102
            259: 'EVENT_P_REV_IN_USE_CHANGE', # 0x103
            260: 'EVENT_SID_CHANGE', # 0x104
            261: 'EVENT_NID_CHANGE', # 0x105
            262: 'EVENT_PZID_CHANGE', # 0x106
            263: 'EVENT_PDE_SESSION_END', # 0x107
            264: 'EVENT_OP_MODE_CHANGE', # 0x108
            265: 'EVENT_MESSAGE_RECEIVED', # 0x109
            266: 'EVENT_MESSAGE_TRANSMITTED', # 0x10a
            267: 'EVENT_TIMER_EXPIRED', # 0x10b
            268: 'EVENT_COUNTER_THRESHOLD', # 0x10c
            269: 'EVENT_CALL_PROCESSING_STATE_CHANGE', # 0x10d
            270: 'EVENT_CALL_CONTROL_INSTANTIATED', # 0x10e
            271: 'EVENT_CALL_CONTROL_STATE_CHANGE', # 0x10f
            272: 'EVENT_CALL_CONTROL_TERMINATED', # 0x110
            273: 'EVENT_REG_ZONE_CHANGE', # 0x111
            274: 'EVENT_SLOTTED_MODE_OPERATION', # 0x112
            275: 'EVENT_QPCH_IN_USE', # 0x113
            276: 'EVENT_IDLE_HANDOFF', # 0x114
            277: 'EVENT_ACCESS_HANDOFF', # 0x115
            278: 'EVENT_ACCESS_PROBE_HANDOFF', # 0x116
            279: 'EVENT_SOFT_HANDOFF', # 0x117
            280: 'EVENT_HARD_HANDOFF_FREQ_CHANGE', # 0x118
            281: 'EVENT_HARD_HANDOFF_FRAME_OFFSET_CHANGE', # 0x119
            282: 'EVENT_HARD_HANDOFF_DISJOINT_ASET', # 0x11a
            283: 'EVENT_UNSUCCESSFUL_HARD_HANDOFF', # 0x11b
            284: 'EVENT_TMSI_ASSIGNED', # 0x11c
            285: 'EVENT_SERVICE_NEGOTIATION_COMPLETED', # 0x11d
            286: 'EVENT_SO_NEGOTIATION_COMPLETED', # 0x11e
            287: 'EVENT_ENTER_CONTROL_HOLD_MODE', # 0x11f
            288: 'EVENT_EXIT_CONTROL_HOLD_MODE', # 0x120
            289: 'EVENT_START_FWD_SUPP_BURST_ASSGN', # 0x121
            290: 'EVENT_END_FWD_SUPP_BURST_ASSGN', # 0x122
            291: 'EVENT_START_REV_SUPP_BURST_ASSGN', # 0x123
            292: 'EVENT_END_REV_SUPP_BURST_ASSGN', # 0x124
            293: 'EVENT_DTX', # 0x125
            294: 'EVENT_T_ADD_ABORT', # 0x126
            295: 'EVENT_CH_IND_CHANGE', # 0x127
            296: 'EVENT_TRANSMITTER_DISABLED', # 0x128
            297: 'EVENT_TRANSMITTER_ENABLED', # 0x129
            298: 'EVENT_SMS_RECEIVED', # 0x12a
            299: 'EVENT_SMS_SENT', # 0x12b
            300: 'EVENT_INACTIVITY_TIMER_EXPIRED', # 0x12c
            301: 'EVENT_DORMANT_TIMER_EXPIRED', # 0x12d
            302: 'EVENT_ACCESS_ATTEMPT_FAIL_MAX_PROBES_SENT', # 0x12e
            303: 'EVENT_ACCESS_ATTEMPT_FAIL_LOSS_OF_PC_OR_FCCC', # 0x12f
            304: 'EVENT_PCH_ACQUIRED', # 0x130
            305: 'EVENT_BCCH_ACQUIRED', # 0x131
            306: 'EVENT_FFCH_ACQUIRED', # 0x132
            307: 'EVENT_FDCCH_ACQUIRED', # 0x133
            308: 'EVENT_FFCH_PLUS_DCCH_ACQUIRED', # 0x134
            309: 'EVENT_REGISTRATION_PERFORMED', # 0x135
            310: 'EVENT_NEW_SYSTEM_IDLE_HANDOFF', # 0x136
            311: 'EVENT_SYSTEM_RESELECTION', # 0x137
            312: 'EVENT_RESCAN', # 0x138
            313: 'EVENT_PROTOCOL_MISMATCH', # 0x139
            314: 'EVENT_LOCK', # 0x13a
            315: 'EVENT_UNLOCK', # 0x13b
            316: 'EVENT_ACCESS_DENIED', # 0x13c
            317: 'EVENT_NDSS_OFF', # 0x13d
            318: 'EVENT_RELEASE', # 0x13e
            319: 'EVENT_ERROR', # 0x13f
            320: 'EVENT_REDIRECTION', # 0x140
            321: 'EVENT_REGISTRATION_REJECTED', # 0x141
            322: 'EVENT_WRONG_SYSTEM', # 0x142
            323: 'EVENT_WRONG_NETWORK', # 0x143
            324: 'EVENT_LOSS_OF_ACQ_AFTER_SLEEP', # 0x144
            325: 'EVENT_POWER_DOWN', # 0x145
            326: 'EVENT_CALL_RELEASE_REQUEST', # 0x146
            327: 'EVENT_SERVICE_INACTIVE', # 0x147
            328: 'EVENT_EXTENDED_RELEASE', # 0x148
            329: 'EVENT_HDR_MSG_RX', # 0x149
            330: 'EVENT_HDR_RXMSG_IGNORED_STATE', # 0x14a
            331: 'EVENT_HDR_RXMSG_IGNORED_SEQ', # 0x14b
            332: 'EVENT_HDR_TXMSG_ACKED', # 0x14c
            333: 'EVENT_HDR_TXMSG_DROPPED', # 0x14d
            334: 'EVENT_HDR_STATE_CHANGE', # 0x14e
            335: 'EVENT_HDR_ALMP_OBEYING_REDIRECTION', # 0x14f
            336: 'EVENT_HDR_ALMP_CONNECTION_CLOSED', # 0x150
            337: 'EVENT_HDR_ALMP_T_SD_RESELECT', # 0x151
            338: 'EVENT_HDR_ALMP_CONNECTION_OPENED', # 0x152
            339: 'EVENT_HDR_HMP_QUEUED_MSG', # 0x153
            340: 'EVENT_HDR_HMP_SENT_MSG', # 0x154
            341: 'EVENT_HDR_HMP_ABORTING_ACMAC_ACTIVATION', # 0x155
            342: 'EVENT_HDR_IDLE_T_CONFIG_RSP', # 0x156
            343: 'EVENT_HDR_IDLE_T_AT_SETUP', # 0x157
            344: 'EVENT_HDR_IDLE_T_SUSPEND', # 0x158
            345: 'EVENT_HDR_IDLE_CONNECTION_DENIED', # 0x159
            346: 'EVENT_HDR_INIT_T_SYNC_ACQ', # 0x15a
            347: 'EVENT_HDR_INIT_PROTOCOL_MISMATCH', # 0x15b
            348: 'EVENT_HDR_OVHD_INFO_CURRENT', # 0x15c
            349: 'EVENT_HDR_OVHD_T_QC_SUPERVISION', # 0x15d
            350: 'EVENT_HDR_OVHD_T_SP_SUPERVISION', # 0x15e
            351: 'EVENT_HDR_OVHD_T_AP_SUPERVISION', # 0x15f
            352: 'EVENT_HDR_OVHD_IGNORED_MSG_UNEXPECTED_LINK', # 0x160
            353: 'EVENT_HDR_OVHD_IGNORED_SP_MSG_DIFF_SEC_SIG', # 0x161
            354: 'EVENT_HDR_OVHD_IGNORED_AP_MSG_DIFF_ACC_SIG', # 0x162
            355: 'EVENT_HDR_OVHD_IGNORED_SP_MSG_DIFF_SEC_ID', # 0x163
            356: 'EVENT_HDR_OVHD_SP_MSG_RX', # 0x164
            357: 'EVENT_HDR_OVHD_AP_MSG_RX', # 0x165
            358: 'EVENT_HDR_RUP_T_CONNECTION_SETUP', # 0x166
            359: 'EVENT_HDR_SLP_MAX_RETRIES', # 0x167
            360: 'EVENT_HDR_LMAC_ACQ_FAIL_PILOT', # 0x168
            361: 'EVENT_HDR_LMAC_ACQ_SUCCESS', # 0x169
            362: 'EVENT_HDR_LMAC_NETWORK_LOST', # 0x16a
            363: 'EVENT_HDR_LMAC_IDLE_HO', # 0x16b
            364: 'EVENT_HDR_LMAC_CHAN_CHANGE_COMPLETE', # 0x16c
            365: 'EVENT_HDR_LMAC_ACCESS_HO_NEEDED', # 0x16d
            366: 'EVENT_HDR_LMAC_ACCESS_HO_COMPLETE', # 0x16e
            367: 'EVENT_HDR_LMAC_ACQUIRE', # 0x16f
            368: 'EVENT_HDR_LMAC_CHANGING_CC_HASH', # 0x170
            369: 'EVENT_HDR_LMAC_IDLE_CHAN_CHANGE', # 0x171
            370: 'EVENT_HDR_CMAC_T_SUPERVISION', # 0x172
            371: 'EVENT_HDR_AMAC_START_ACCESS', # 0x173
            372: 'EVENT_HDR_AMAC_PROBING_STOPPED', # 0x174
            373: 'EVENT_HDR_AMAC_ACCESS_COMPLETE', # 0x175
            374: 'EVENT_HDR_AMAC_ACCESS_ABORTED', # 0x176
            375: 'EVENT_HDR_AMAC_MAX_PROBES', # 0x177
            376: 'EVENT_HDR_FMAC_DROP_PKT', # 0x178
            377: 'EVENT_HDR_RMAC_T_RATE_LIMIT', # 0x179
            378: 'EVENT_HDR_RMAC_TX_STARTED', # 0x17a
            379: 'EVENT_HDR_RMAC_TX_STOPPED', # 0x17b
            380: 'EVENT_HDR_SMP_T_KEEP_ALIVE', # 0x17c
            381: 'EVENT_HDR_AMP_ASSIGN_MSG_IGNORED_FRESH', # 0x17d
            382: 'EVENT_HDR_AMP_T_AT_RESPONSE', # 0x17e
            383: 'EVENT_HDR_AMP_T_DUAL_ADDRESS', # 0x17f
            384: 'EVENT_HDR_SCP_BEGIN_CONFIGURATION', # 0x180
            385: 'EVENT_HDR_SCP_T_CONFIG_RSP', # 0x181
            386: 'EVENT_HDR_SCP_T_AN_INIT_STATE', # 0x182
            387: 'EVENT_WCDMA_L1_STATE', # 0x183
            388: 'EVENT_WCDMA_IMSI', # 0x184
            389: 'EVENT_GSM_L1_STATE', # 0x185
            390: 'EVENT_RANDOM_ACCESS_REQUEST', # 0x186
            391: 'EVENT_HIGH_LEVEL_CALL_PROCESSING_STATE_CHANGE', # 0x187
            392: 'EVENT_ENCRYPTION_FAILURE', # 0x188
            393: 'EVENT_ACCT_BLOCKED', # 0x189
            394: 'EVENT_COMMON_CHANNEL_MONITORED', # 0x18a
            395: 'EVENT_SOFT_HANDOFF_V2', # 0x18b
            396: 'EVENT_HARD_HANDOFF_FREQ_CHANGE_V2', # 0x18c
            397: 'EVENT_HARD_HANDOFF_FRAME_OFFSET_CHANGE_V2', # 0x18d
            398: 'EVENT_HARD_HANDOFF_DISJOINT_ASET_V2', # 0x18e
            399: 'EVENT_WCDMA_NEW_REFERENCE_CELL', # 0x18f
            400: 'EVENT_CALL_CONTROL_CONREF_CHANGE', # 0x190
            401: 'EVENT_GPS_SESSION_BEGIN', # 0x191
            402: 'EVENT_GPS_SESSION_END', # 0x192
            403: 'EVENT_GPS_WAITING_ON_SA', # 0x193
            404: 'EVENT_GPS_PPM_START', # 0x194
            405: 'EVENT_GPS_PPM_RESULTS', # 0x195
            406: 'EVENT_GPS_PPM_END', # 0x196
            407: 'EVENT_GPS_VISIT_BEGIN', # 0x197
            408: 'EVENT_GPS_VISIT_END', # 0x198
            409: 'EVENT_GPS_CDMA_RESUMED_AFTER_GPS_VISIT', # 0x199
            410: 'EVENT_GPS_PD_SESSION_BEGIN', # 0x19a
            411: 'EVENT_GPS_PD_SESSION_END', # 0x19b
            412: 'EVENT_GPS_IS801_RX', # 0x19c
            413: 'EVENT_GPS_IS801_TX', # 0x19d
            414: 'EVENT_POWERUP', # 0x19e
            415: 'EVENT_WCDMA_ASET', # 0x19f
            416: 'EVENT_CM_CALL_STATE', # 0x1a0
            417: 'EVENT_CM_OPERATIONAL_MODE', # 0x1a1
            418: 'EVENT_CM_SYSTEM_MODE', # 0x1a2
            419: 'EVENT_DEEP_SLEEP', # 0x1a3
            420: 'EVENT_WAKEUP', # 0x1a4
            421: 'EVENT_ACQUISITION_MODE', # 0x1a5
            422: 'EVENT_ACQUISITION_TYPE', # 0x1a6
            423: 'EVENT_ACP_EXIT', # 0x1a7
            424: 'EVENT_CDMA_EXIT', # 0x1a8
            425: 'EVENT_HDR_HYBRID_POWER_SAVE', # 0x1a9
            426: 'EVENT_HDR_DEEP_SLEEP', # 0x1aa
            427: 'EVENT_HDR_RESELECTION', # 0x1ab
            428: 'EVENT_SAM_LOCK_GRANTED', # 0x1ac
            429: 'EVENT_SAM_LOCK_RELEASED', # 0x1ad
            430: 'EVENT_GSM_HANDOVER_START', # 0x1ae
            431: 'EVENT_GSM_HANDOVER_END', # 0x1af
            432: 'EVENT_GSM_LINK_FAILURE', # 0x1b0
            433: 'EVENT_GSM_RESELECT_START', # 0x1b1
            434: 'EVENT_GSM_RESELECT_END', # 0x1b2
            435: 'EVENT_GSM_CAMP_ATTEMPT_START', # 0x1b3
            436: 'EVENT_GSM_RR_IN_SERVICE', # 0x1b4
            437: 'EVENT_GSM_RR_OUT_OF_SERVICE', # 0x1b5
            438: 'EVENT_GSM_PAGE_RECEIVED', # 0x1b6
            439: 'EVENT_GSM_CAMP_ATTEMPT_END', # 0x1b7
            440: 'EVENT_GPS_IS801_TIMEOUT', # 0x1b8
            441: 'EVENT_GPS_IS801_DISCARD', # 0x1b9
            442: 'EVENT_GSM_CELL_SELECTION_START', # 0x1ba
            443: 'EVENT_GSM_CELL_SELECTION_END', # 0x1bb
            444: 'EVENT_GSM_POWER_SCAN_STATUS', # 0x1bc
            445: 'EVENT_GSM_PLMN_LIST_START', # 0x1bd
            446: 'EVENT_GSM_PLMN_LIST_END', # 0x1be
            447: 'EVENT_WCDMA_INTER_RAT_HANDOVER_START', # 0x1bf
            448: 'EVENT_WCDMA_INTER_RAT_HANDOVER_END', # 0x1c0
            449: 'EVENT_GSM_MESSAGE_SENT', # 0x1c1
            451: 'EVENT_GSM_TIMER_EXPIRED', # 0x1c3
            452: 'EVENT_GSM_COUNTER_EXPIRED', # 0x1c4
            453: 'EVENT_NAS_MESSAGE_SENT', # 0x1c5
            454: 'EVENT_NAS_MESSAGE_RECEIVED', # 0x1c6
            455: 'EVENT_RRC_MESSAGE_SENT', # 0x1c7
            456: 'EVENT_RRC_MESSAGE_RECEIVED', # 0x1c8
            457: 'EVENT_CAMERA_CANNOT_CAPTURE', # 0x1c9
            458: 'EVENT_CAMERA_CANNOT_CONFIG_JPEG', # 0x1ca
            459: 'EVENT_CAMERA_CANNOT_CONFIG_VFE', # 0x1cb
            460: 'EVENT_CAMERA_CANNOT_ENCODE', # 0x1cc
            461: 'EVENT_CAMERA_CANNOT_IDLE_DSP', # 0x1cd
            462: 'EVENT_CAMERA_CANNOT_LOAD_DSP', # 0x1ce
            463: 'EVENT_CAMERA_DSP_FATAL', # 0x1cf
            464: 'EVENT_CAMERA_DSP_REQ_ILLEGAL', # 0x1d0
            465: 'EVENT_CAMERA_EFS_FAILED', # 0x1d1
            466: 'EVENT_CAMERA_EXIT', # 0x1d2
            467: 'EVENT_CAMERA_FORMAT_NOT_SUPPORTED', # 0x1d3
            468: 'EVENT_CAMERA_FUNCTION_REJECTED', # 0x1d4
            469: 'EVENT_CAMERA_IMAGE_CORRUPT', # 0x1d5
            470: 'EVENT_CAMERA_INVALID_CONFIG_PARM', # 0x1d6
            471: 'EVENT_CAMERA_INVALID_SET_ID', # 0x1d7
            472: 'EVENT_CAMERA_INVALID_STATE', # 0x1d8
            473: 'EVENT_CAMERA_JPEG_ENCODED', # 0x1d9
            474: 'EVENT_CAMERA_NO_MEMORY', # 0x1da
            475: 'EVENT_CAMERA_NO_PICTURE', # 0x1db
            476: 'EVENT_CAMERA_PICTURE_SAVED', # 0x1dc
            477: 'EVENT_CAMERA_PICTURE_TAKEN', # 0x1dd
            478: 'EVENT_CAMERA_PREVIEW', # 0x1de
            479: 'EVENT_CAMERA_RECORD', # 0x1df
            480: 'EVENT_CAMERA_SAVE_PICTURE', # 0x1e0
            481: 'EVENT_CAMERA_SET_FAILED', # 0x1e1
            482: 'EVENT_CAMERA_SET_SUCCEEDED', # 0x1e2
            483: 'EVENT_CAMERA_START', # 0x1e3
            484: 'EVENT_CAMERA_STOP', # 0x1e4
            485: 'EVENT_CAMERA_TAKE_PICTURE', # 0x1e5
            486: 'EVENT_DIAG_STRESS_TEST_NO_PAYLOAD', # 0x1e6
            487: 'EVENT_DIAG_STRESS_TEST_WITH_PAYLOAD', # 0x1e7
            488: 'EVENT_CM_CALL_ORIG_START_P1', # 0x1e8
            489: 'EVENT_CM_CALL_ORIG_START_P2', # 0x1e9
            490: 'EVENT_CM_CALL_ORIG_START_P3', # 0x1ea
            491: 'EVENT_CM_CALL_ORIG_SWITCH_TO_HDR', # 0x1eb
            492: 'EVENT_CM_CALL_ORIG_REDIAL', # 0x1ec
            493: 'EVENT_CM_CALL_ORIG_SEND_HDR_ORIG', # 0x1ed
            494: 'EVENT_CM_CALL_ORIG_SEND_MC_ORIG', # 0x1ee
            495: 'EVENT_CM_CALL_ORIG_END', # 0x1ef
            496: 'EVENT_CM_CALL_ORIG_CONNECTED', # 0x1f0
            497: 'EVENT_MT_SMS_NOTIFY', # 0x1f1
            498: 'EVENT_SMS_SLOT_WAKEUP', # 0x1f2
            499: 'EVENT_MO_SMS_STATUS', # 0x1f3
            500: 'EVENT_GPRS_SURROUND_SEARCH_START', # 0x1f4
            501: 'EVENT_GPRS_SURROUND_SEARCH_END', # 0x1f5
            502: 'EVENT_GPRS_MAC_RESELECT_IND', # 0x1f6
            503: 'EVENT_GPRS_PAGE_RECEIVED', # 0x1f7
            504: 'EVENT_GPRS_LINK_FAILURE', # 0x1f8
            505: 'EVENT_GPRS_CELL_UPDATE_START', # 0x1f9
            506: 'EVENT_GPRS_CELL_UPDATE_END', # 0x1fa
            507: 'EVENT_GPRS_EARLY_CAMPING', # 0x1fb
            508: 'EVENT_PACKET_RANDOM_ACCESS_REQ', # 0x1fc
            509: 'EVENT_GPRS_MAC_MSG_SENT', # 0x1fd
            510: 'EVENT_GPRS_MAC_MSG_RECEIVED', # 0x1fe
            511: 'EVENT_GPRS_SMGMM_MSG_SENT', # 0x1ff
            512: 'EVENT_GPRS_SMGMM_MSG_RECEIVED', # 0x200
            513: 'EVENT_CP_MATCHED_MSG', # 0x201
            514: 'EVENT_PREF_SYS_RESEL', # 0x202
            515: 'EVENT_WCDMA_LAYER1_PRACH', # 0x203
            516: 'EVENT_WCDMA_LAYER1_MEASUREMENT', # 0x204
            517: 'EVENT_MOBILITY_MANAGEMENT_STATE_CHANGE', # 0x205
            518: 'EVENT_LSM_STATE_CHANGE', # 0x206
            519: 'EVENT_RLP', # 0x207
            520: 'EVENT_CM_MODE_PREF', # 0x208
            521: 'EVENT_CM_BAND_PREF', # 0x209
            522: 'EVENT_CM_ROAM_PREF', # 0x20a
            523: 'EVENT_CM_SRV_DOMAIN_PREF', # 0x20b
            524: 'EVENT_CM_GW_ACQ_ORDER_PREF', # 0x20c
            525: 'EVENT_CM_HYBRID_PREF', # 0x20d
            526: 'EVENT_CM_NETWORK_SEL_MODE_PREF', # 0x20e
            527: 'EVENT_WCDMA_L1_SUSPEND', # 0x20f
            528: 'EVENT_WCDMA_L1_RESUME', # 0x210
            529: 'EVENT_WCDMA_L1_STOPPED', # 0x211
            530: 'EVENT_WCDMA_TO_WCDMA_RESELECTION_START', # 0x212
            531: 'EVENT_WCDMA_TO_GSM_RESELECTION_START', # 0x213
            532: 'EVENT_WCDMA_TO_GSM_RESELECTION_END', # 0x214
            533: 'EVENT_WCDMA_TO_WCDMA_RESELECTION_END', # 0x215
            534: 'EVENT_WCDMA_RACH_ATTEMPT', # 0x216
            535: 'EVENT_START_FWD_SUPP_BURST_ASSIGN', # 0x217
            536: 'EVENT_START_REV_SUPP_BURST_ASSIGN', # 0x218
            537: 'EVENT_REV_FCH_GATING_IN_USE', # 0x219
            538: 'EVENT_PPP', # 0x21a
            539: 'EVENT_MIP', # 0x21b
            540: 'EVENT_TCP', # 0x21c
            541: 'EVENT_CAMERA_EXIF_FAILED', # 0x21d
            542: 'EVENT_CAMERA_VIDEO_FAILED', # 0x21e
            543: 'EVENT_CAMERA_NO_SENSOR', # 0x21f
            544: 'EVENT_CAMERA_ABORT', # 0x220
            545: 'EVENT_CM_BLOCK_HDR_ORIG_DURING_GPS', # 0x221
            546: 'EVENT_CM_ALLOW_HDR_ORIG_DURING_GPS', # 0x222
            547: 'EVENT_GSM_AMR_STATE_CHANGE', # 0x223
            548: 'EVENT_GSM_RATSCCH_IN_DTX', # 0x224
            549: 'EVENT_GSM_FACCH_IN_DTX', # 0x225
            550: 'EVENT_GSM_FACCH_AND_RATSCCH_COLLISION', # 0x226
            551: 'EVENT_GSM_FACCH_AND_SID_UPDATE_COLLISION', # 0x227
            552: 'EVENT_GSM_RATSCCH_AND_SID_UPDATE_COLLISION', # 0x228
            553: 'EVENT_GSM_RATSCCH_CMI_PHASE_CHANGE', # 0x229
            554: 'EVENT_GSM_RATSCCH_REQ_ACT_TIMER_EXPIRY', # 0x22a
            555: 'EVENT_GSM_RATSCCH_ACK_ACT_TIMER_EXPIRY', # 0x22b
            556: 'EVENT_GSM_AMR_CMC_TURNAROUND_TIME', # 0x22c
            557: 'EVENT_CM_PLMN_FOUND', # 0x22d
            558: 'EVENT_CM_SERVICE_CONFIRMED', # 0x22e
            559: 'EVENT_GPRS_MAC_CAMPED_ON_CELL', # 0x22f
            560: 'EVENT_GPRS_LLC_READY_TIMER_START', # 0x230
            561: 'EVENT_GPRS_LLC_READY_TIMER_END', # 0x231
            562: 'EVENT_WCDMA_PHYCHAN_ESTABLISHED', # 0x232
            563: 'EVENT_HS_DISPLAY_BMP_CAPTURE_STATUS', # 0x233
            564: 'EVENT_WCDMA_CELL_SELECTED', # 0x234
            565: 'EVENT_WCDMA_PAGE_RECEIVED', # 0x235
            566: 'EVENT_WCDMA_SEND_KEY', # 0x236
            567: 'EVENT_WCDMA_RL_FAILURE', # 0x237
            568: 'EVENT_WCDMA_MAX_RESET', # 0x238
            569: 'EVENT_WCDMA_CALL_SETUP', # 0x239
            570: 'EVENT_WCDMA_CALL_DROPPED', # 0x23a
            571: 'EVENT_WCDMA_RRC_STATE', # 0x23b
            572: 'EVENT_GPS_PD_CONNECTION_TIMEOUT', # 0x23c
            573: 'EVENT_GPS_PD_DISCONNECTION_COMPLETE', # 0x23d
            574: 'EVENT_MEDIA_PLAYER_START', # 0x23e
            575: 'EVENT_MEDIA_PLAYER_STOP', # 0x23f
            576: 'EVENT_MEDIA_PLAYER_SEEK', # 0x240
            577: 'EVENT_GPS_SRCH_START', # 0x241
            578: 'EVENT_GPS_SRCH_END', # 0x242
            579: 'EVENT_GPS_PPM_PAUSE', # 0x243
            580: 'EVENT_GPS_PPM_RESUME', # 0x244
            581: 'EVENT_GPS_SA_RECEIVED', # 0x245
            582: 'EVENT_GPS_CLK_ON', # 0x246
            583: 'EVENT_GPS_CLK_OFF', # 0x247
            584: 'EVENT_GPS_VISIT_REQUEST', # 0x248
            585: 'EVENT_GPS_VISIT_RESPONSE', # 0x249
            586: 'EVENT_GPS_TA_START', # 0x24a
            587: 'EVENT_GPS_DSP_READY', # 0x24b
            588: 'EVENT_GPS_DSP_CHANNEL_START', # 0x24c
            589: 'EVENT_GPS_DSP_CHANNEL_DONE', # 0x24d
            590: 'EVENT_GPS_DSP_STOP', # 0x24e
            591: 'EVENT_GPS_DSP_DONE', # 0x24f
            592: 'EVENT_GPS_TB_END', # 0x250
            593: 'EVENT_GPS_SRCH_LARGE_DOPP_WIN', # 0x251
            594: 'EVENT_GPS_SRCH_EXCEPTION', # 0x252
            595: 'EVENT_GPS_SRCH_HW_POLLING1', # 0x253
            596: 'EVENT_GPS_SRCH_HW_POLLING2', # 0x254
            597: 'EVENT_GPS_PGI_ACTION_PROCESS', # 0x255
            598: 'EVENT_GPS_GSC_ACTION_PROCESS', # 0x256
            599: 'EVENT_GPS_PGI_ABORT', # 0x257
            600: 'EVENT_GPS_GSC_ABORT', # 0x258
            601: 'EVENT_GPS_PD_FIX_START', # 0x259
            602: 'EVENT_GPS_PD_FIX_END', # 0x25a
            603: 'EVENT_GPS_DATA_DOWNLOAD_START', # 0x25b
            604: 'EVENT_GPS_DATA_DOWNLOAD_END', # 0x25c
            605: 'EVENT_GPS_PD_SESSION_START', # 0x25d
            606: 'EVENT_GPS_DORMANCY_BEGIN', # 0x25e
            607: 'EVENT_GPS_DORMANCY_END', # 0x25f
            608: 'EVENT_GPS_PRQ_TIMEOUT', # 0x260
            609: 'EVENT_GPS_PD_CONNECTION_START', # 0x261
            610: 'EVENT_GPS_PD_CONNECTION_ESTABLISHED', # 0x262
            611: 'EVENT_GPS_PD_DISCONNECTION_START', # 0x263
            612: 'EVENT_GPS_FTEST_FIX_START', # 0x264
            613: 'EVENT_GPS_FTEST_FIX_END', # 0x265
            614: 'EVENT_GPS_PD_POSITION', # 0x266
            615: 'EVENT_GPS_E911_START', # 0x267
            616: 'EVENT_GPS_E911_END', # 0x268
            617: 'EVENT_GPS_DBM_SEND_FAILURE', # 0x269
            618: 'EVENT_GPS_UAPDMS_STATE_CHANGE', # 0x26a
            619: 'EVENT_WCDMA_OUT_OF_SERVICE', # 0x26b
            620: 'EVENT_GSM_L1_SUBSTATE', # 0x26c
            621: 'EVENT_SD_EVENT_ACTION', # 0x26d
            622: 'EVENT_SD_EVENT_ACTION_HYBR', # 0x26e
            623: 'EVENT_UMTS_CALLS_STATISTICS', # 0x26f
            624: 'EVENT_PZID_HAT_STARTED', # 0x270
            625: 'EVENT_WCDMA_DRX_CYCLE', # 0x271
            626: 'EVENT_WCDMA_RE_ACQUISITION_FAIL', # 0x272
            627: 'EVENT_WCDMA_RRC_RB0_SETUP_FAILURE', # 0x273
            628: 'EVENT_WCDMA_RRC_PHYCHAN_EST_FAILURE', # 0x274
            629: 'EVENT_CM_CALL_EVENT_ORIG', # 0x275
            630: 'EVENT_CM_CALL_EVENT_CONNECT', # 0x276
            631: 'EVENT_CM_CALL_EVENT_END', # 0x277
            632: 'EVENT_CM_ENTER_EMERGENCY_CB', # 0x278
            633: 'EVENT_CM_EXIT_EMERGENCY_CB', # 0x279
            634: 'EVENT_PZID_HAT_EXPIRED', # 0x27a
            635: 'EVENT_HDR_SMP_SESSION_CLOSED', # 0x27b
            636: 'EVENT_WCDMA_MEMORY_LEAK', # 0x27c
            637: 'EVENT_PZID_HT_STARTED', # 0x27d
            638: 'EVENT_PZID_HT_EXPIRED', # 0x27e
            639: 'EVENT_ACCESS_ENTRY_HANDOFF', # 0x27f
            640: 'EVENT_BREW_APP_START', # 0x280
            641: 'EVENT_BREW_APP_STOP', # 0x281
            642: 'EVENT_BREW_APP_PAUSE', # 0x282
            643: 'EVENT_BREW_APP_RESUME', # 0x283
            644: 'EVENT_BREW_EXT_MODULE_START', # 0x284
            645: 'EVENT_BREW_EXT_MODULE_STOP', # 0x285
            646: 'EVENT_BREW_ERROR', # 0x286
            647: 'EVENT_BREW_RESERVED_647', # 0x287
            648: 'EVENT_BREW_RESERVED_648', # 0x288
            649: 'EVENT_BREW_RESERVED_649', # 0x289
            650: 'EVENT_BREW_RESERVED_650', # 0x28a
            651: 'EVENT_BREW_RESERVED_651', # 0x28b
            652: 'EVENT_BREW_RESERVED_652', # 0x28c
            653: 'EVENT_BREW_RESERVED_653', # 0x28d
            654: 'EVENT_BREW_RESERVED_654', # 0x28e
            655: 'EVENT_BREW_RESERVED_655', # 0x28f
            656: 'EVENT_BREW_USER_656', # 0x290
            657: 'EVENT_BREW_GENERIC', # 0x291
            658: 'EVENT_BREW_MEDIAPLAYER_SELECT_FILE', # 0x292
            659: 'EVENT_BREW_MEDIAPLAYER_CONTROL', # 0x293
            660: 'EVENT_BREW_APP_FORMITEM_STACK_CHANGE', # 0x294
            661: 'EVENT_BREW_CATAPP_RECV_PROACTIVE_CMD', # 0x295
            662: 'EVENT_BREW_CATAPP_TERMINAL_RSP', # 0x296
            663: 'EVENT_BREW_CATAPP_NO_DISPLAY', # 0x297
            664: 'EVENT_BREW_SIRIUS_EMAIL_DELETE', # 0x298
            665: 'EVENT_BREW_SIRIUS_EMAIL_OPERATION_COMPLETE', # 0x299
            666: 'EVENT_BREW_SIRIUS_EMAIL_NEW_EMAIL_NOTIFICATION', # 0x29a
            667: 'EVENT_BREW_UNDEFINED_667', # 0x29b
            668: 'EVENT_BREW_UNDEFINED_668', # 0x29c
            669: 'EVENT_BREW_UNDEFINED_669', # 0x29d
            670: 'EVENT_BREW_UNDEFINED_670', # 0x29e
            671: 'EVENT_BREW_UNDEFINED_671', # 0x29f
            672: 'EVENT_BREW_UNDEFINED_672', # 0x2a0
            673: 'EVENT_BREW_UNDEFINED_673', # 0x2a1
            674: 'EVENT_BREW_UNDEFINED_674', # 0x2a2
            675: 'EVENT_BREW_UNDEFINED_675', # 0x2a3
            676: 'EVENT_BREW_UNDEFINED_676', # 0x2a4
            677: 'EVENT_BREW_UNDEFINED_677', # 0x2a5
            678: 'EVENT_BREW_UNDEFINED_678', # 0x2a6
            679: 'EVENT_BREW_UNDEFINED_679', # 0x2a7
            680: 'EVENT_BREW_UNDEFINED_680', # 0x2a8
            681: 'EVENT_BREW_UNDEFINED_681', # 0x2a9
            682: 'EVENT_BREW_UNDEFINED_682', # 0x2aa
            683: 'EVENT_BREW_UNDEFINED_683', # 0x2ab
            684: 'EVENT_BREW_UNDEFINED_684', # 0x2ac
            685: 'EVENT_BREW_UNDEFINED_685', # 0x2ad
            686: 'EVENT_BREW_UNDEFINED_686', # 0x2ae
            687: 'EVENT_BREW_UNDEFINED_687', # 0x2af
            688: 'EVENT_BREW_UNDEFINED_688', # 0x2b0
            689: 'EVENT_BREW_UNDEFINED_689', # 0x2b1
            690: 'EVENT_BREW_UNDEFINED_690', # 0x2b2
            691: 'EVENT_BREW_UNDEFINED_691', # 0x2b3
            692: 'EVENT_BREW_UNDEFINED_692', # 0x2b4
            693: 'EVENT_BREW_UNDEFINED_693', # 0x2b5
            694: 'EVENT_BREW_UNDEFINED_694', # 0x2b6
            695: 'EVENT_BREW_UNDEFINED_695', # 0x2b7
            696: 'EVENT_BREW_UNDEFINED_696', # 0x2b8
            697: 'EVENT_BREW_UNDEFINED_697', # 0x2b9
            698: 'EVENT_BREW_UNDEFINED_698', # 0x2ba
            699: 'EVENT_BREW_UNDEFINED_699', # 0x2bb
            700: 'EVENT_BREW_UNDEFINED_700', # 0x2bc
            701: 'EVENT_BREW_UNDEFINED_701', # 0x2bd
            702: 'EVENT_BREW_UNDEFINED_702', # 0x2be
            703: 'EVENT_BREW_UNDEFINED_703', # 0x2bf
            704: 'EVENT_BREW_UNDEFINED_704', # 0x2c0
            705: 'EVENT_BREW_UNDEFINED_705', # 0x2c1
            706: 'EVENT_BREW_UNDEFINED_706', # 0x2c2
            707: 'EVENT_BREW_UNDEFINED_707', # 0x2c3
            708: 'EVENT_BREW_UNDEFINED_708', # 0x2c4
            709: 'EVENT_BREW_UNDEFINED_709', # 0x2c5
            710: 'EVENT_BREW_UNDEFINED_710', # 0x2c6
            711: 'EVENT_BREW_UNDEFINED_711', # 0x2c7
            712: 'EVENT_BREW_UNDEFINED_712', # 0x2c8
            713: 'EVENT_BREW_UNDEFINED_713', # 0x2c9
            714: 'EVENT_BREW_UNDEFINED_714', # 0x2ca
            715: 'EVENT_BREW_UNDEFINED_715', # 0x2cb
            716: 'EVENT_BREW_UNDEFINED_716', # 0x2cc
            717: 'EVENT_BREW_UNDEFINED_717', # 0x2cd
            718: 'EVENT_BREW_UNDEFINED_718', # 0x2ce
            719: 'EVENT_BREW_UNDEFINED_719', # 0x2cf
            720: 'EVENT_BREW_UNDEFINED_720', # 0x2d0
            721: 'EVENT_BREW_UNDEFINED_721', # 0x2d1
            722: 'EVENT_BREW_UNDEFINED_722', # 0x2d2
            723: 'EVENT_BREW_UNDEFINED_723', # 0x2d3
            724: 'EVENT_BREW_UNDEFINED_724', # 0x2d4
            725: 'EVENT_BREW_UNDEFINED_725', # 0x2d5
            726: 'EVENT_BREW_UNDEFINED_726', # 0x2d6
            727: 'EVENT_BREW_UNDEFINED_727', # 0x2d7
            728: 'EVENT_BREW_UNDEFINED_728', # 0x2d8
            729: 'EVENT_BREW_UNDEFINED_729', # 0x2d9
            730: 'EVENT_BREW_UNDEFINED_730', # 0x2da
            731: 'EVENT_BREW_UNDEFINED_731', # 0x2db
            732: 'EVENT_BREW_UNDEFINED_732', # 0x2dc
            733: 'EVENT_BREW_UNDEFINED_733', # 0x2dd
            734: 'EVENT_BREW_UNDEFINED_734', # 0x2de
            735: 'EVENT_BREW_UNDEFINED_735', # 0x2df
            736: 'EVENT_BREW_UNDEFINED_736', # 0x2e0
            737: 'EVENT_BREW_UNDEFINED_737', # 0x2e1
            738: 'EVENT_BREW_UNDEFINED_738', # 0x2e2
            739: 'EVENT_BREW_UNDEFINED_739', # 0x2e3
            740: 'EVENT_BREW_UNDEFINED_740', # 0x2e4
            741: 'EVENT_BREW_UNDEFINED_741', # 0x2e5
            742: 'EVENT_BREW_UNDEFINED_742', # 0x2e6
            743: 'EVENT_BREW_UNDEFINED_743', # 0x2e7
            744: 'EVENT_BREW_UNDEFINED_744', # 0x2e8
            745: 'EVENT_BREW_UNDEFINED_745', # 0x2e9
            746: 'EVENT_BREW_UNDEFINED_746', # 0x2ea
            747: 'EVENT_BREW_UNDEFINED_747', # 0x2eb
            748: 'EVENT_BREW_UNDEFINED_748', # 0x2ec
            749: 'EVENT_BREW_UNDEFINED_749', # 0x2ed
            750: 'EVENT_BREW_UNDEFINED_750', # 0x2ee
            751: 'EVENT_BREW_UNDEFINED_751', # 0x2ef
            752: 'EVENT_BREW_UNDEFINED_752', # 0x2f0
            753: 'EVENT_BREW_UNDEFINED_753', # 0x2f1
            754: 'EVENT_BREW_UNDEFINED_754', # 0x2f2
            755: 'EVENT_BREW_UNDEFINED_755', # 0x2f3
            756: 'EVENT_BREW_UNDEFINED_756', # 0x2f4
            757: 'EVENT_BREW_UNDEFINED_757', # 0x2f5
            758: 'EVENT_BREW_UNDEFINED_758', # 0x2f6
            759: 'EVENT_BREW_UNDEFINED_759', # 0x2f7
            760: 'EVENT_BREW_UNDEFINED_760', # 0x2f8
            761: 'EVENT_BREW_UNDEFINED_761', # 0x2f9
            762: 'EVENT_BREW_UNDEFINED_762', # 0x2fa
            763: 'EVENT_BREW_UNDEFINED_763', # 0x2fb
            764: 'EVENT_BREW_UNDEFINED_764', # 0x2fc
            765: 'EVENT_BREW_UNDEFINED_765', # 0x2fd
            766: 'EVENT_BREW_UNDEFINED_766', # 0x2fe
            767: 'EVENT_BREW_UNDEFINED_767', # 0x2ff
            768: 'EVENT_BREW_UNDEFINED_768', # 0x300
            769: 'EVENT_BREW_UNDEFINED_769', # 0x301
            770: 'EVENT_BREW_UNDEFINED_770', # 0x302
            771: 'EVENT_BREW_UNDEFINED_771', # 0x303
            772: 'EVENT_BREW_UNDEFINED_772', # 0x304
            773: 'EVENT_BREW_UNDEFINED_773', # 0x305
            774: 'EVENT_BREW_UNDEFINED_774', # 0x306
            775: 'EVENT_BREW_UNDEFINED_775', # 0x307
            776: 'EVENT_BREW_UNDEFINED_776', # 0x308
            777: 'EVENT_BREW_UNDEFINED_777', # 0x309
            778: 'EVENT_BREW_UNDEFINED_778', # 0x30a
            779: 'EVENT_BREW_UNDEFINED_779', # 0x30b
            780: 'EVENT_BREW_UNDEFINED_780', # 0x30c
            781: 'EVENT_BREW_UNDEFINED_781', # 0x30d
            782: 'EVENT_BREW_UNDEFINED_782', # 0x30e
            783: 'EVENT_BREW_UNDEFINED_783', # 0x30f
            784: 'EVENT_BREW_UNDEFINED_784', # 0x310
            785: 'EVENT_BREW_UNDEFINED_785', # 0x311
            786: 'EVENT_BREW_UNDEFINED_786', # 0x312
            787: 'EVENT_BREW_UNDEFINED_787', # 0x313
            788: 'EVENT_BREW_UNDEFINED_788', # 0x314
            789: 'EVENT_BREW_UNDEFINED_789', # 0x315
            790: 'EVENT_BREW_UNDEFINED_790', # 0x316
            791: 'EVENT_BREW_UNDEFINED_791', # 0x317
            792: 'EVENT_BREW_UNDEFINED_792', # 0x318
            793: 'EVENT_BREW_UNDEFINED_793', # 0x319
            794: 'EVENT_BREW_UNDEFINED_794', # 0x31a
            795: 'EVENT_BREW_UNDEFINED_795', # 0x31b
            796: 'EVENT_BREW_UNDEFINED_796', # 0x31c
            797: 'EVENT_BREW_UNDEFINED_797', # 0x31d
            798: 'EVENT_BREW_UNDEFINED_798', # 0x31e
            799: 'EVENT_BREW_UNDEFINED_799', # 0x31f
            800: 'EVENT_BREW_UNDEFINED_800', # 0x320
            801: 'EVENT_BREW_UNDEFINED_801', # 0x321
            802: 'EVENT_BREW_UNDEFINED_802', # 0x322
            803: 'EVENT_BREW_UNDEFINED_803', # 0x323
            804: 'EVENT_BREW_UNDEFINED_804', # 0x324
            805: 'EVENT_BREW_UNDEFINED_805', # 0x325
            806: 'EVENT_BREW_UNDEFINED_806', # 0x326
            807: 'EVENT_BREW_UNDEFINED_807', # 0x327
            808: 'EVENT_BREW_UNDEFINED_808', # 0x328
            809: 'EVENT_BREW_UNDEFINED_809', # 0x329
            810: 'EVENT_BREW_UNDEFINED_810', # 0x32a
            811: 'EVENT_BREW_UNDEFINED_811', # 0x32b
            812: 'EVENT_BREW_UNDEFINED_812', # 0x32c
            813: 'EVENT_BREW_UNDEFINED_813', # 0x32d
            814: 'EVENT_BREW_UNDEFINED_814', # 0x32e
            815: 'EVENT_BREW_UNDEFINED_815', # 0x32f
            816: 'EVENT_BREW_UNDEFINED_816', # 0x330
            817: 'EVENT_BREW_UNDEFINED_817', # 0x331
            818: 'EVENT_BREW_UNDEFINED_818', # 0x332
            819: 'EVENT_BREW_UNDEFINED_819', # 0x333
            820: 'EVENT_BREW_UNDEFINED_820', # 0x334
            821: 'EVENT_BREW_UNDEFINED_821', # 0x335
            822: 'EVENT_BREW_UNDEFINED_822', # 0x336
            823: 'EVENT_BREW_UNDEFINED_823', # 0x337
            824: 'EVENT_BREW_UNDEFINED_824', # 0x338
            825: 'EVENT_BREW_UNDEFINED_825', # 0x339
            826: 'EVENT_BREW_UNDEFINED_826', # 0x33a
            827: 'EVENT_BREW_UNDEFINED_827', # 0x33b
            828: 'EVENT_BREW_UNDEFINED_828', # 0x33c
            829: 'EVENT_BREW_UNDEFINED_829', # 0x33d
            830: 'EVENT_BREW_UNDEFINED_830', # 0x33e
            831: 'EVENT_BREW_UNDEFINED_831', # 0x33f
            832: 'EVENT_BREW_UNDEFINED_832', # 0x340
            833: 'EVENT_BREW_UNDEFINED_833', # 0x341
            834: 'EVENT_BREW_UNDEFINED_834', # 0x342
            835: 'EVENT_BREW_UNDEFINED_835', # 0x343
            836: 'EVENT_BREW_UNDEFINED_836', # 0x344
            837: 'EVENT_BREW_UNDEFINED_837', # 0x345
            838: 'EVENT_BREW_UNDEFINED_838', # 0x346
            839: 'EVENT_BREW_UNDEFINED_839', # 0x347
            840: 'EVENT_BREW_UNDEFINED_840', # 0x348
            841: 'EVENT_BREW_UNDEFINED_841', # 0x349
            842: 'EVENT_BREW_UNDEFINED_842', # 0x34a
            843: 'EVENT_BREW_UNDEFINED_843', # 0x34b
            844: 'EVENT_BREW_UNDEFINED_844', # 0x34c
            845: 'EVENT_BREW_UNDEFINED_845', # 0x34d
            846: 'EVENT_BREW_UNDEFINED_846', # 0x34e
            847: 'EVENT_BREW_UNDEFINED_847', # 0x34f
            848: 'EVENT_BREW_UNDEFINED_848', # 0x350
            849: 'EVENT_BREW_UNDEFINED_849', # 0x351
            850: 'EVENT_BREW_UNDEFINED_850', # 0x352
            851: 'EVENT_BREW_UNDEFINED_851', # 0x353
            852: 'EVENT_BREW_UNDEFINED_852', # 0x354
            853: 'EVENT_BREW_UNDEFINED_853', # 0x355
            854: 'EVENT_BREW_UNDEFINED_854', # 0x356
            855: 'EVENT_BREW_UNDEFINED_855', # 0x357
            856: 'EVENT_BREW_UNDEFINED_856', # 0x358
            857: 'EVENT_BREW_UNDEFINED_857', # 0x359
            858: 'EVENT_BREW_UNDEFINED_858', # 0x35a
            859: 'EVENT_BREW_UNDEFINED_859', # 0x35b
            860: 'EVENT_BREW_UNDEFINED_860', # 0x35c
            861: 'EVENT_BREW_UNDEFINED_861', # 0x35d
            862: 'EVENT_BREW_UNDEFINED_862', # 0x35e
            863: 'EVENT_BREW_UNDEFINED_863', # 0x35f
            864: 'EVENT_BREW_UNDEFINED_864', # 0x360
            865: 'EVENT_BREW_UNDEFINED_865', # 0x361
            866: 'EVENT_BREW_UNDEFINED_866', # 0x362
            867: 'EVENT_BREW_UNDEFINED_867', # 0x363
            868: 'EVENT_BREW_UNDEFINED_868', # 0x364
            869: 'EVENT_BREW_UNDEFINED_869', # 0x365
            870: 'EVENT_BREW_UNDEFINED_870', # 0x366
            871: 'EVENT_BREW_UNDEFINED_871', # 0x367
            872: 'EVENT_BREW_UNDEFINED_872', # 0x368
            873: 'EVENT_BREW_UNDEFINED_873', # 0x369
            874: 'EVENT_BREW_UNDEFINED_874', # 0x36a
            875: 'EVENT_BREW_UNDEFINED_875', # 0x36b
            876: 'EVENT_BREW_UNDEFINED_876', # 0x36c
            877: 'EVENT_BREW_UNDEFINED_877', # 0x36d
            878: 'EVENT_BREW_UNDEFINED_878', # 0x36e
            879: 'EVENT_BREW_UNDEFINED_879', # 0x36f
            880: 'EVENT_BREW_UNDEFINED_880', # 0x370
            881: 'EVENT_BREW_UNDEFINED_881', # 0x371
            882: 'EVENT_BREW_UNDEFINED_882', # 0x372
            883: 'EVENT_BREW_UNDEFINED_883', # 0x373
            884: 'EVENT_BREW_UNDEFINED_884', # 0x374
            885: 'EVENT_BREW_UNDEFINED_885', # 0x375
            886: 'EVENT_BREW_UNDEFINED_886', # 0x376
            887: 'EVENT_BREW_UNDEFINED_887', # 0x377
            888: 'EVENT_BREW_UNDEFINED_888', # 0x378
            889: 'EVENT_BREW_UNDEFINED_889', # 0x379
            890: 'EVENT_BREW_UNDEFINED_890', # 0x37a
            891: 'EVENT_BREW_UNDEFINED_891', # 0x37b
            892: 'EVENT_BREW_UNDEFINED_892', # 0x37c
            893: 'EVENT_BREW_UNDEFINED_893', # 0x37d
            894: 'EVENT_BREW_UNDEFINED_894', # 0x37e
            895: 'EVENT_BREW_UNDEFINED_895', # 0x37f
            896: 'EVENT_WCDMA_PS_DATA_RATE', # 0x380
            897: 'EVENT_GSM_TO_WCDMA_RESELECT_END', # 0x381
            898: 'EVENT_PZID_HAI_ENABLED', # 0x382
            899: 'EVENT_PZID_HAI_DISABLED', # 0x383
            900: 'EVENT_GSM_TO_WCDMA_HANDOVER_START', # 0x384
            901: 'EVENT_WCDMA_RRC_MODE', # 0x385
            902: 'EVENT_WCDMA_L1_ACQ_SUBSTATE', # 0x386
            903: 'EVENT_WCDMA_PHYCHAN_CFG_CHANGED', # 0x387
            904: 'EVENT_QTV_CLIP_STARTED', # 0x388
            905: 'EVENT_QTV_CLIP_ENDED', # 0x389
            906: 'EVENT_QTV_SDP_PARSER_REJECT', # 0x38a
            907: 'EVENT_QTV_CLIP_PAUSE', # 0x38b
            908: 'EVENT_QTV_CLIP_REPOSITIONING', # 0x38c
            909: 'EVENT_QTV_CLIP_ZOOM_IN', # 0x38d
            910: 'EVENT_QTV_CLIP_ZOOM_OUT', # 0x38e
            911: 'EVENT_QTV_CLIP_ROTATE', # 0x38f
            912: 'EVENT_QTV_CLIP_PAUSE_RESUME', # 0x390
            913: 'EVENT_QTV_CLIP_REPOSITION_RESUME', # 0x391
            914: 'EVENT_QTV_DSP_INIT', # 0x392
            915: 'EVENT_QTV_STREAMING_SERVER_URL', # 0x393
            916: 'EVENT_QTV_SERVER_PORTS_USED', # 0x394
            917: 'EVENT_QTV_USING_PROXY_SERVER', # 0x395
            918: 'EVENT_QTV_STREAMER_STATE_IDLE', # 0x396
            919: 'EVENT_QTV_STREAMER_STATE_CONNECTING', # 0x397
            920: 'EVENT_QTV_STREAMER_STATE_SETTING_TRACKS', # 0x398
            921: 'EVENT_QTV_STREAMER_STATE_STREAMING', # 0x399
            922: 'EVENT_QTV_STREAMER_STATE_PAUSED', # 0x39a
            923: 'EVENT_QTV_STREAMER_STATE_SUSPENDED', # 0x39b
            924: 'EVENT_QTV_STREAMER_CONNECTED', # 0x39c
            925: 'EVENT_QTV_STREAMER_INITSTREAM_FAIL', # 0x39d
            926: 'EVENT_QTV_BUFFERING_STARTED', # 0x39e
            927: 'EVENT_QTV_BUFFERING_ENDED', # 0x39f
            928: 'EVENT_QTV_CLIP_FULLSCREEN', # 0x3a0
            929: 'EVENT_QTV_PS_DOWNLOAD_STARTED', # 0x3a1
            930: 'EVENT_QTV_PSEUDO_STREAM_STARTED', # 0x3a2
            931: 'EVENT_QTV_PS_PLAYER_STATE_PSEUDO_PAUSE', # 0x3a3
            932: 'EVENT_QTV_PS_PLAYER_STATE_PSEUDO_RESUME', # 0x3a4
            933: 'EVENT_QTV_PARSER_STATE_READY', # 0x3a5
            934: 'EVENT_QTV_FRAGMENT_PLAYBACK_BEGIN', # 0x3a6
            935: 'EVENT_QTV_FRAGMENT_PLAYBACK_COMPLETE', # 0x3a7
            936: 'EVENT_QTV_PARSER_STATE_PSEUDO_PAUSE', # 0x3a8
            937: 'EVENT_QTV_PLAYER_STATE_PSEUDO_PAUSE', # 0x3a9
            938: 'EVENT_QTV_PARSER_STATE_PSEUDO_RESUME', # 0x3aa
            939: 'EVENT_QTV_PLAYER_STATE_PSEUDO_RESUME', # 0x3ab
            940: 'EVENT_QTV_FRAGMENTED_FILE_DECODE_START', # 0x3ac
            941: 'EVENT_QTV_FRAGMENTED_FILE_END_SUCCESS', # 0x3ad
            942: 'EVENT_QTV_DOWNLOAD_DATA_REPORT', # 0x3ae
            943: 'EVENT_QTV_VDEC_DIAG_DECODE_CALLBACK', # 0x3af
            944: 'EVENT_QTV_URL_PLAYED_IS_MULTICAST', # 0x3b0
            945: 'EVENT_QTV_VDEC_DIAG_STATUS', # 0x3b1
            946: 'EVENT_QTV_STREAMING_URL_OPEN', # 0x3b2
            947: 'EVENT_QTV_STREAMING_URL_OPENING', # 0x3b3
            948: 'EVENT_QTV_CLIP_ENDED_VER2', # 0x3b4
            949: 'EVENT_QTV_SILENCE_INSERTION_STARTED', # 0x3b5
            950: 'EVENT_QTV_SILENCE_INSERTION_ENDED', # 0x3b6
            951: 'EVENT_QTV_AUDIO_CHANNEL_SWITCH_FRAME', # 0x3b7
            952: 'EVENT_QTV_FIRST_VIDEO_FRAME_RENDERED', # 0x3b8
            953: 'EVENT_QTV_FIRST_VIDEO_I_FRAME_RENDERED', # 0x3b9
            954: 'EVENT_QTV_SDP_SELECTED', # 0x3ba
            955: 'EVENT_QTV_DIAG_PLAYER_STATUS', # 0x3bb
            956: 'EVENT_QTV_SILENCE_INSERTION_DURATION', # 0x3bc
            957: 'EVENT_QTV_UNDEFINED_957', # 0x3bd
            958: 'EVENT_QTV_UNDEFINED_958', # 0x3be
            959: 'EVENT_QTV_UNDEFINED_959', # 0x3bf
            960: 'EVENT_QTV_UNDEFINED_960', # 0x3c0
            961: 'EVENT_QTV_UNDEFINED_961', # 0x3c1
            962: 'EVENT_QTV_UNDEFINED_962', # 0x3c2
            963: 'EVENT_QTV_UNDEFINED_963', # 0x3c3
            964: 'EVENT_QTV_UNDEFINED_964', # 0x3c4
            965: 'EVENT_QTV_UNDEFINED_965', # 0x3c5
            966: 'EVENT_QTV_UNDEFINED_966', # 0x3c6
            967: 'EVENT_QTV_UNDEFINED_967', # 0x3c7
            968: 'EVENT_DS_SETS_ARM_CLOCK_FASTER', # 0x3c8
            969: 'EVENT_DS_SETS_ARM_CLOCK_SLOWER', # 0x3c9
            970: 'EVENT_SMS_STATISTICS', # 0x3ca
            971: 'EVENT_SM_PDP_STATE', # 0x3cb
            972: 'EVENT_MVS_STATE', # 0x3cc
            973: 'EVENT_SECSSL', # 0x3cd
            974: 'EVENT_SECTEST', # 0x3ce
            975: 'EVENT_SECVPN', # 0x3cf
            976: 'EVENT_SECCRYPT', # 0x3d0
            977: 'EVENT_SECCRYPT_CMD', # 0x3d1
            978: 'EVENT_SEC_RESERVED_978', # 0x3d2
            979: 'EVENT_SEC_RESERVED_979', # 0x3d3
            980: 'EVENT_SEC_RESERVED_980', # 0x3d4
            981: 'EVENT_SEC_RESERVED_981', # 0x3d5
            982: 'EVENT_ARM_CLK_FREQUENCY_CHANGE', # 0x3d6
            983: 'EVENT_ADSP_CLK_FREQUENCY_CHANGE', # 0x3d7
            984: 'EVENT_MDSP_CLK_FREQUENCY_CHANGE', # 0x3d8
            985: 'EVENT_CELL_CHANGE_INDICATION', # 0x3d9
            986: 'EVENT_CB_STATE_CHANGE', # 0x3da
            987: 'EVENT_SMSCB_L1_STATE_CHANGE', # 0x3db
            988: 'EVENT_SMSCB_L1_COLLISION', # 0x3dc
            989: 'EVENT_WMS_SEARCH_REQUEST', # 0x3dd
            990: 'EVENT_CM_GET_PASSWORD_IND', # 0x3de
            991: 'EVENT_CM_PASSWORD_AUTHENTICATION_STATUS', # 0x3df
            992: 'EVENT_CM_USS_RESPONSE_NOTIFY_IND', # 0x3e0
            993: 'EVENT_CM_USS_CONF', # 0x3e1
            994: 'EVENT_CM_RELEASE_USS_IND', # 0x3e2
            995: 'EVENT_CM_FWD_AOC_IND', # 0x3e3
            996: 'EVENT_PZID_ID', # 0x3e4
            997: 'EVENT_PZID_HT_VALUE', # 0x3e5
            998: 'EVENT_PZID_EXISTS_IN_LIST', # 0x3e6
            999: 'EVENT_GSDI_GET_FILE_ATTRIBUTES', # 0x3e7
            1000: 'EVENT_GSDI_SIM_READ', # 0x3e8
            1001: 'EVENT_GSDI_SIM_WRITE', # 0x3e9
            1002: 'EVENT_GSDI_GET_PIN_STATUS', # 0x3ea
            1003: 'EVENT_GSDI_VERIFY_PIN', # 0x3eb
            1004: 'EVENT_GSDI_UNBLOCK_PIN', # 0x3ec
            1005: 'EVENT_GSDI_DISABLE_PIN', # 0x3ed
            1006: 'EVENT_GSDI_ENABLE_PIN', # 0x3ee
            1007: 'EVENT_GSDI_SIM_INCREASE', # 0x3ef
            1008: 'EVENT_GSDI_EXECUTE_APDU_REQ', # 0x3f0
            1009: 'EVENT_SEG_UPM_ADDR_MISMATCH', # 0x3f1
            1010: 'EVENT_WCDMA_PRACH', # 0x3f2
            1011: 'EVENT_GSDI_SELECT', # 0x3f3
            1012: 'EVENT_WCDMA_RAB_RATE_RECONFIG', # 0x3f4
            1013: 'EVENT_WCDMA_RLC_RESETS', # 0x3f5
            1014: 'EVENT_WCDMA_RLC_OPEN_CLOSE', # 0x3f6
            1015: 'EVENT_WCDMA_RLC_MRW', # 0x3f7
            1016: 'EVENT_QVP_APP_PROCESS_EVENT', # 0x3f8
            1017: 'EVENT_QVP_APP_STATE_CHANGED_EVENT', # 0x3f9
            1018: 'EVENT_QVP_APP_CALL_CONNECTED_EVENT', # 0x3fa
            1019: 'EVENT_GSDI_CARD_EVENT_NOTIFICATION', # 0x3fb
            1020: 'EVENT_CM_DATA_AVAILABLE', # 0x3fc
            1021: 'EVENT_CM_DS_INTERRAT_STATE', # 0x3fd
            1022: 'EVENT_MM_STATE', # 0x3fe
            1023: 'EVENT_GMM_STATE', # 0x3ff
            1024: 'EVENT_PLMN_INFORMATION', # 0x400
            1025: 'EVENT_COREAPP_SET_VOICE_PRIVACY', # 0x401
            1026: 'EVENT_COREAPP_GET_VOICE_PRIVACY', # 0x402
            1027: 'EVENT_HARD_HANDOFF_LONG_CODE_MASK_CHANGE', # 0x403
            1028: 'EVENT_VCTCXO_FREEZE', # 0x404
            1029: 'EVENT_VCTCXO_UNFREEZE', # 0x405
            1030: 'EVENT_SMS_SLOT_WAKEUP_V2', # 0x406
            1031: 'EVENT_QVP_RCVD_FIRST_VIDEO_FRAME', # 0x407
            1032: 'EVENT_QVP_CALL_RELEASED', # 0x408
            1033: 'EVENT_CB_SMS_NOTIFY', # 0x409
            1034: 'EVENT_GPS_PDSM_EVENT_REPORT', # 0x40a
            1035: 'EVENT_LONG_CODE_MASK_CHANGED', # 0x40b
            1036: 'EVENT_DS707', # 0x40c
            1037: 'EVENT_GSDI_ACTIVATE_FEATURE_IND', # 0x40d
            1038: 'EVENT_GSDI_DEACTIVATE_FEATURE_IND', # 0x40e
            1039: 'EVENT_GSDI_GET_FEATURE_IND', # 0x40f
            1040: 'EVENT_GSDI_SET_FEATURE_DATA', # 0x410
            1041: 'EVENT_GSDI_UNBLOCK_FEATURE_IND', # 0x411
            1042: 'EVENT_GSDI_GET_CONTROL_KEY', # 0x412
            1043: 'EVENT_GSDI_OTA_DEPERSO', # 0x413
            1044: 'EVENT_GSDI_GET_PERM_FEATURE_IND', # 0x414
            1045: 'EVENT_GSDI_PERM_DISBALE_FEATURE_IND', # 0x415
            1046: 'EVENT_GSM_L1_VOCODER_INITIALIZE', # 0x416
            1047: 'EVENT_GSM_L1_ALIGN_VFR', # 0x417
            1048: 'EVENT_GSM_L1_VOCODER_ENABLED', # 0x418
            1049: 'EVENT_HDR_AMAC_PERSISTENCE_FAILED', # 0x419
            1050: 'EVENT_HDR_AMAC_PERSISTENCE_PASSED', # 0x41a
            1051: 'EVENT_MFLO_STREAM_STATE', # 0x41b
            1052: 'EVENT_MFLO_CONTROL_CHANNEL_STATE_CHANGE', # 0x41c
            1053: 'EVENT_MFLO_SLEEP_STATE_CHANGE', # 0x41d
            1054: 'EVENT_MFLO_NETWORK_STATE_CHANGE', # 0x41e
            1055: 'EVENT_MFLO_TRANS_STATE', # 0x41f
            1056: 'EVENT_MFLO_OIS_STATE', # 0x420
            1057: 'EVENT_MFLO_RXD_STATE', # 0x421
            1058: 'EVENT_MFLO_HIPRI_STATE_CHANGE', # 0x422
            1059: 'EVENT_MFLO_CAS_STATE', # 0x423
            1060: 'EVENT_MFLO_ACQ_STATE', # 0x424
            1061: 'EVENT_MFLO_OSCAR_FRAME_DECODED', # 0x425
            1062: 'EVENT_MFLO_CHAN_SWITCH_RENDERED', # 0x426
            1063: 'EVENT_MFLO_OSCAR_DEC_EXCEPTION_DETECTED', # 0x427
            1064: 'EVENT_MFLO_MFN_SUBSTATE', # 0x428
            1065: 'EVENT_MFLO_MFN_STATE', # 0x429
            1066: 'EVENT_MFLO_MFN_VERTICAL_HANDOFF', # 0x42a
            1067: 'EVENT_MFLO_MFN_ACQ_STATE', # 0x42b
            1068: 'EVENT_MFLO_FLOW_STATUS', # 0x42c
            1069: 'EVENT_MFLO_NETWORK_STATUS', # 0x42d
            1070: 'EVENT_MFLO_UNDEFINED_1070', # 0x42e
            1071: 'EVENT_CM_LCS_MOLR_CONF', # 0x42f
            1072: 'EVENT_PPP_NETMODEL', # 0x430
            1073: 'EVENT_CAMERA_PROFILING', # 0x431
            1074: 'EVENT_MAC_HS_T1_EXPIRY', # 0x432
            1075: 'EVENT_ASYNC_DS707', # 0x433
            1076: 'EVENT_PKT_DS707', # 0x434
            1077: 'EVENT_GPRS_TIMER_EXPIRY', # 0x435
            1078: 'EVENT_GPRS_MAC_IDLE_IND', # 0x436
            1079: 'EVENT_GPRS_PACKET_CHANNEL_REQUEST', # 0x437
            1080: 'EVENT_GPRS_ACCESS_REJECT', # 0x438
            1081: 'EVENT_GPRS_PACKET_RESOURCE_REQUEST', # 0x439
            1082: 'EVENT_GPRS_PACKET_UPLINK_ASSIGNMENT', # 0x43a
            1083: 'EVENT_GPRS_PACKET_DOWNLINK_ASSIGNMENT', # 0x43b
            1084: 'EVENT_PACKET_TIMESLOT_RECONFIGURE', # 0x43c
            1085: 'EVENT_GPRS_TBF_RELEASE', # 0x43d
            1086: 'EVENT_GPRS_CELL_CHANGE_ORDER', # 0x43e
            1087: 'EVENT_GPRS_CELL_CHANGE_FAILURE', # 0x43f
            1088: 'EVENT_GSM_AMR_RATSCCH_REQ', # 0x440
            1089: 'EVENT_GSM_AMR_RATSCCH_RSP', # 0x441
            1090: 'EVENT_SD_SRV_IND_HYBR_WLAN', # 0x442
            1091: 'EVENT_SD_EVENT_ACTION_HYBR_WLAN', # 0x443
            1092: 'EVENT_GPS_PD_DEMOD_SESS_START', # 0x444
            1093: 'EVENT_GPS_PD_DEMOD_SESS_END', # 0x445
            1094: 'EVENT_GPS_SV_ACQUIRED', # 0x446
            1095: 'EVENT_GPS_SV_BIT_EDGE_FOUND', # 0x447
            1096: 'EVENT_GPS_DEMOD_STARTED', # 0x448
            1097: 'EVENT_GPS_DEMOD_OUT_OF_LOCK', # 0x449
            1098: 'EVENT_GPS_DEMOD_STOPPED', # 0x44a
            1099: 'EVENT_GPS_DEMOD_PREAMBLE_FOUND', # 0x44b
            1100: 'EVENT_GPS_DEMOD_FRAME_SYNC_STATUS', # 0x44c
            1101: 'EVENT_GPS_DEMOD_SUBFRAME', # 0x44d
            1102: 'EVENT_GPS_DEMOD_EPHEMERIS_COMPLETE', # 0x44e
            1103: 'EVENT_GPS_DEMOD_ALMANAC_COMPLETE', # 0x44f
            1104: 'EVENT_GPS_DEMOD_BIT_EDGE_STATUS', # 0x450
            1105: 'EVENT_RAT_CHANGE', # 0x451
            1106: 'EVENT_REGISTRATION_SUPPRESSED', # 0x452
            1107: 'EVENT_HDR_RUP_DIST_BASED_REG', # 0x453
            1108: 'EVENT_GPS_DIAG_APP_TRACKING_START', # 0x454
            1109: 'EVENT_GPS_DIAG_APP_TRACKING_END', # 0x455
            1110: 'EVENT_GPS_DIAG_APP_POSITION_SUCCESS', # 0x456
            1111: 'EVENT_GPS_DIAG_APP_POSITION_FAILURE', # 0x457
            1112: 'EVENT_GSM_AMR_MULTIRATE_IE', # 0x458
            1113: 'EVENT_EPZID_HYSTERESIS_ENABLED', # 0x459
            1114: 'EVENT_EPZID_HYSTERESIS_DISABLED', # 0x45a
            1115: 'EVENT_EPZID_HT_STARTED', # 0x45b
            1116: 'EVENT_EPZID_HT_EXPIRED', # 0x45c
            1117: 'EVENT_HDR_BCMCS_FLOW_STATE_CHANGE', # 0x45d
            1118: 'EVENT_HDR_LMAC_UPDATE_BC_STATUS', # 0x45e
            1119: 'EVENT_DS_CAM_TIMER', # 0x45f
            1120: 'EVENT_DS_RDUD_TIMER', # 0x460
            1121: 'EVENT_DS_CTA_TIMER', # 0x461
            1122: 'EVENT_DS_FALLBACK', # 0x462
            1123: 'EVENT_DS3G_CAM_FLOW_CTRL_TIMER', # 0x463
            1124: 'EVENT_GPS_JAMMER_DETECTION_TEST_PASS', # 0x464
            1125: 'EVENT_GPS_JAMMER_DETECTION_TEST_FAILURE', # 0x465
            1126: 'EVENT_JAMMER_DETECT_NOISE_STATS', # 0x466
            1127: 'EVENT_GPS_GET_PARAM', # 0x467
            1128: 'EVENT_GPS_GET_PARAM_BS_INFO', # 0x468
            1129: 'EVENT_HS_SERVING_CELL_CHANGE', # 0x469
            1130: 'EVENT_HS_DSCH_STATUS', # 0x46a
            1131: 'EVENT_SMGMM_REQUEST_SENT', # 0x46b
            1132: 'EVENT_SMGMM_REJECT_RECEIVED', # 0x46c
            1133: 'EVENT_LINUX_APP_STOP', # 0x46d
            1134: 'EVENT_GPS_PD_CME_SESSION_START', # 0x46e
            1135: 'EVENT_GPS_PD_CME_SESSION_END', # 0x46f
            1136: 'EVENT_SIP_REGISTER_START', # 0x470
            1137: 'EVENT_SIP_REGISTER_DONE', # 0x471
            1138: 'EVENT_SIP_CALL_SETUP_START', # 0x472
            1139: 'EVENT_SIP_CALL_SETUP_DONE', # 0x473
            1140: 'EVENT_SIP_CALL_RELEASE_START', # 0x474
            1141: 'EVENT_SIP_CALL_RELEASE_DONE', # 0x475
            1142: 'EVENT_AUDIO_FRAME_SENT_TO_DECODER', # 0x476
            1143: 'EVENT_VIDEO_FRAME_SENT_TO_DECODER', # 0x477
            1144: 'EVENT_DEC_RENDER_FRAME', # 0x478
            1145: 'EVENT_DEC_RENDER_DONE', # 0x479
            1146: 'EVENT_DEC_START_DECODING', # 0x47a
            1147: 'EVENT_DEC_FRAME_DECODED', # 0x47b
            1148: 'EVENT_V_ENCODED', # 0x47c
            1149: 'EVENT_DEC_START_DECODING_EXT', # 0x47d
            1150: 'EVENT_DEC_FRAME_DECODED_EXT', # 0x47e
            1151: 'EVENT_QVIDEOPHONE_UNDEFINED_1151', # 0x47f
            1152: 'EVENT_QVIDEOPHONE_UNDEFINED_1152', # 0x480
            1153: 'EVENT_QVIDEOPHONE_UNDEFINED_1153', # 0x481
            1154: 'EVENT_QVIDEOPHONE_UNDEFINED_1154', # 0x482
            1155: 'EVENT_QVIDEOPHONE_UNDEFINED_1155', # 0x483
            1156: 'EVENT_GPS_CME_POS_REQ', # 0x484
            1157: 'EVENT_GPS_CME_FIX_START', # 0x485
            1158: 'EVENT_GPS_CME_FIX_END', # 0x486
            1159: 'EVENT_GPS_SEED_CLM', # 0x487
            1160: 'EVENT_GPS_SEED_SID', # 0x488
            1161: 'EVENT_GPS_SEED_SL', # 0x489
            1162: 'EVENT_GPS_SEED_GET', # 0x48a
            1163: 'EVENT_HDR_OVHD_BC_MSG_RX', # 0x48b
            1164: 'EVENT_HDR_OVHD_T_BC_SUPERVISION', # 0x48c
            1165: 'EVENT_HDR_LMAC_SET_BCMCS_PAGE_CYCLE', # 0x48d
            1166: 'EVENT_HDR_HMP_SESSION_CLOSED', # 0x48e
            1167: 'EVENT_WLAN_CP', # 0x48f
            1168: 'EVENT_ARP', # 0x490
            1169: 'EVENT_DHCP', # 0x491
            1170: 'EVENT_WLAN_WPA', # 0x492
            1171: 'EVENT_EAP', # 0x493
            1172: 'EVENT_LAN_1X', # 0x494
            1173: 'EVENT_CAMERA_SVCS_START', # 0x495
            1174: 'EVENT_CAMERA_SVCS_STOP', # 0x496
            1175: 'EVENT_BCMCS_SRVC_AVAILABLE', # 0x497
            1176: 'EVENT_BCMCS_SRVC_LOST', # 0x498
            1177: 'EVENT_BCMCS_FLOW_REGISTERED', # 0x499
            1178: 'EVENT_BCMCS_FLOW_DEREGISTERED', # 0x49a
            1179: 'EVENT_BCMCS_FLOW_STATUS_CHANGED', # 0x49b
            1180: 'EVENT_CAMERA_SVCS_X', # 0x49c
            1181: 'EVENT_CM_CALL_EVENT_ORIG_THR', # 0x49d
            1182: 'EVENT_VFE_MSG_CONFIG_COMPLETE', # 0x49e
            1183: 'EVENT_VFE_MSG_IDLE_COMPLETE', # 0x49f
            1184: 'EVENT_VFE_MSG_UPDATE_COMPLETE', # 0x4a0
            1185: 'EVENT_VFE_MSG_AE_AWB_STATS', # 0x4a1
            1186: 'EVENT_DSP_VIDEO_ENC_DOWNLOAD_DONE', # 0x4a2
            1187: 'EVENT_DSP_VIDEO_ENC_SELECTION_DONE', # 0x4a3
            1188: 'EVENT_DSP_VIDEO_ENC_CONFIG_DONE', # 0x4a4
            1189: 'EVENT_DSP_VIDEO_ENC_FRAME_DONE', # 0x4a5
            1190: 'EVENT_HDR_OVHD_BCMCS_CHAN_CHANGE', # 0x4a6
            1191: 'EVENT_QVS_REGISTER_START', # 0x4a7
            1192: 'EVENT_QVS_REGISTER_DONE', # 0x4a8
            1193: 'EVENT_QVS_REGISTER_FAILED', # 0x4a9
            1194: 'EVENT_QVS_CALL_SETUP_START', # 0x4aa
            1195: 'EVENT_QVS_CALL_SETUP_DONE', # 0x4ab
            1196: 'EVENT_QVS_CALL_SETUP_FAILED', # 0x4ac
            1197: 'EVENT_QVS_CALL_RELEASE_START', # 0x4ad
            1198: 'EVENT_QVS_CALL_RELEASE_DONE', # 0x4ae
            1199: 'EVENT_QVS_CALL_RELEASE_FAILED', # 0x4af
            1200: 'EVENT_CAMCORDER_START_RECORD', # 0x4b0
            1201: 'EVENT_CAMCORDER_START_TRANSCODE', # 0x4b1
            1202: 'EVENT_CAMCORDER_FRAME_DROP', # 0x4b2
            1203: 'EVENT_CAMCORDER_AUDIODUB', # 0x4b3
            1204: 'EVENT_PSMM_SENT', # 0x4b4
            1205: 'EVENT_GPS_PD_FALLBACK_MODE', # 0x4b5
            1206: 'EVENT_PEAP', # 0x4b6
            1207: 'EVENT_TTLS', # 0x4b7
            1208: 'EVENT_TLS', # 0x4b8
            1209: 'EVENT_WCDMA_TO_WCDMA_RESELECTION_VER2_START', # 0x4b9
            1210: 'EVENT_EUL_RECONFIG_OR_ASU', # 0x4ba
            1211: 'EVENT_EUL_SERVING_CELL_CHANGE', # 0x4bb
            1212: 'EVENT_EUL_PHYSICAL_LAYER_RECONFIG', # 0x4bc
            1213: 'EVENT_DRM_ROAP_TRIGGER_RECIEVED', # 0x4bd
            1214: 'EVENT_DRM_ROAP_PROTOCOL_START', # 0x4be
            1215: 'EVENT_DRM_ROAP_REQUEST', # 0x4bf
            1216: 'EVENT_DRM_ROAP_REQUEST_EXTENSION', # 0x4c0
            1217: 'EVENT_DRM_ROAP_RESPONSE', # 0x4c1
            1218: 'EVENT_DRM_ROAP_RESPONSE_EXTENSION', # 0x4c2
            1219: 'EVENT_DRM_ROAP_RI_CONTEXT', # 0x4c3
            1220: 'EVENT_DRM_ROAP_ERROR', # 0x4c4
            1221: 'EVENT_DRM_ROAP_RSP_VALIDATION', # 0x4c5
            1222: 'EVENT_DRM_ROAP_PROTOCOL_END', # 0x4c6
            1223: 'EVENT_DS_WMK_ALLOCATED', # 0x4c7
            1224: 'EVENT_DS_WMK_DEALLOCATED', # 0x4c8
            1225: 'EVENT_DS_WMK_FLUSHED', # 0x4c9
            1226: 'EVENT_DS_WMK_FLOW_ENABLED', # 0x4ca
            1227: 'EVENT_DS_WMK_FLOW_DISABLED', # 0x4cb
            1228: 'EVENT_HDR_IDLE_SET_SLEEP_DURATION', # 0x4cc
            1229: 'EVENT_HDR_SCM_SESSION_CHANGED', # 0x4cd
            1230: 'EVENT_UMTS_TO_CDMA_DATA_HANDOVER', # 0x4ce
            1231: 'EVENT_UMTS_TO_CDMA_VOICE_HANDOVER', # 0x4cf
            1232: 'EVENT_MO_SMS_RETRY_ATTEMPT', # 0x4d0
            1233: 'EVENT_HDR_LMAC_UPDATE_QSM_STATUS', # 0x4d1
            1234: 'EVENT_CM_CELL_SRV_IND', # 0x4d2
            1235: 'EVENT_RLP_NAK_ABORT', # 0x4d3
            1236: 'EVENT_DRM_RIGHTS_OPERATION', # 0x4d4
            1237: 'EVENT_DS_RESV_MSG_SENT_REV_FLOWS', # 0x4d5
            1238: 'EVENT_DS_RESV_MSG_SENT_FWD_FLOWS', # 0x4d6
            1239: 'EVENT_DS_RESV_RESP_SUCCESS_RECD', # 0x4d7
            1240: 'EVENT_DS_RESV_RESP_FAILURE_RECD', # 0x4d8
            1241: 'EVENT_GPS_PD_COMM_FAILURE', # 0x4d9
            1242: 'EVENT_GPS_PD_COMM_DONE', # 0x4da
            1243: 'EVENT_GPS_PD_EVENT_END', # 0x4db
            1244: 'EVENT_GPS_PA_EVENT_CALLBACK', # 0x4dc
            1245: 'EVENT_GPS_PD_CMD_ERR_CALLBACK', # 0x4dd
            1246: 'EVENT_GPS_PA_CMD_ERR_CALLBACK', # 0x4de
            1247: 'EVENT_GPS_LM_ENTER_SA_RF_VERIF', # 0x4df
            1248: 'EVENT_GPS_LM_EXIT_SA_RF_VERIF', # 0x4e0
            1249: 'EVENT_GPS_LM_ERROR_SA_RF_VERIF', # 0x4e1
            1250: 'EVENT_GPS_LM_PD_COMPLETE', # 0x4e2
            1251: 'EVENT_GPS_LM_IQ_TEST_COMPLETE', # 0x4e3
            1252: 'EVENT_PM_APP_OTG_INIT', # 0x4e4
            1253: 'EVENT_PM_APP_OTG_RESET', # 0x4e5
            1254: 'EVENT_PM_APP_OTG_ACQUIRE_BUS_REQ', # 0x4e6
            1255: 'EVENT_PM_APP_OTG_RELINQUISH_BUS_REQ', # 0x4e7
            1256: 'EVENT_PM_APP_OTG_SUSPEND', # 0x4e8
            1257: 'EVENT_PM_APP_OTG_RESUME', # 0x4e9
            1258: 'EVENT_PM_APP_OTG_DEVICE_ATTACHED', # 0x4ea
            1259: 'EVENT_PM_APP_OTG_DEVICE_DETACHED', # 0x4eb
            1260: 'EVENT_PM_APP_OTG_HOST_MODE_REM_PERI_DIS', # 0x4ec
            1261: 'EVENT_PM_APP_OTG_PERI_MODE_PREPARE_FOR_REM_HOST_WAKEUP_SIG', # 0x4ed
            1262: 'EVENT_PM_APP_OTG_PERI_MODE_REM_HOST_WAKEUP_SIG_DONE', # 0x4ee
            1263: 'EVENT_PM_APP_OTG_SET_REM_WAKEUP_CAPABILITY', # 0x4ef
            1264: 'EVENT_PM_APP_OTG_OPERATIONAL_ERROR', # 0x4f0
            1265: 'EVENT_PM_APP_OTG_CONFIGURE_USB_POWER_CONSUMER', # 0x4f1
            1266: 'EVENT_PM_APP_OTG_SET_USB_POWER_CONSUMPTION_REQUIREMENT', # 0x4f2
            1267: 'EVENT_PM_APP_OTG_PERI_MODE_PROCESS_USB_POWER_LINE_CONT_REQ', # 0x4f3
            1268: 'EVENT_PM_APP_OTG_PERI_MODE_SET_REM_A_DEV_INFO', # 0x4f4
            1269: 'EVENT_PM_APP_OTG_STATE_TRANSITION', # 0x4f5
            1270: 'EVENT_DTV_TABLE_ACQ_SUCCESS', # 0x4f6
            1271: 'EVENT_DTV_TABLE_ACQ_FAIL', # 0x4f7
            1272: 'EVENT_DTV_DVBH_SEL_PLTFM_REQ_RCVD', # 0x4f8
            1273: 'EVENT_DTV_DVBH_PLTFM_ACQ_SUCCESS', # 0x4f9
            1274: 'EVENT_DTV_DVBH_PLTFM_ACQ_FAIL', # 0x4fa
            1275: 'EVENT_DTV_DVBH_TBL_MGR_STATE_CHANGED', # 0x4fb
            1276: 'EVENT_DTV_DVBH_CE_STATE_CHANGED', # 0x4fc
            1277: 'EVENT_DTV_DVBH_MCAST_JOIN_REQ_RCVD', # 0x4fd
            1278: 'EVENT_DTV_DVBH_MCAST_LEAVE_REQ_RCVD', # 0x4fe
            1279: 'EVENT_DTV_DVBH_INIT_REQ_RCVD', # 0x4ff
            1280: 'EVENT_DTV_DVBH_MCAST_JOIN_SUCCESS', # 0x500
            1281: 'EVENT_DTV_DVBH_MCAST_JOIN_FAILURE', # 0x501
            1282: 'EVENT_DTV_DVBH_MCAST_LEAVE_SUCCESS', # 0x502
            1283: 'EVENT_DTV_DVBH_MCAST_LEAVE_FAILURE', # 0x503
            1284: 'EVENT_DTV_DVBH_INIT_SUCCESS', # 0x504
            1285: 'EVENT_DTV_DVBH_INIT_FAILURE', # 0x505
            1286: 'EVENT_GPS_LM_SESSION_START', # 0x506
            1287: 'EVENT_GPS_LM_SESSION_END', # 0x507
            1288: 'EVENT_GPS_LM_FIX_REQUEST_START', # 0x508
            1289: 'EVENT_GPS_LM_FIX_REQUEST_END', # 0x509
            1290: 'EVENT_GPS_LM_PRM_REQUEST_START', # 0x50a
            1291: 'EVENT_GPS_LM_PRM_REQUEST_END', # 0x50b
            1292: 'EVENT_GPS_LM_SESSION_CONTINUE', # 0x50c
            1293: 'EVENT_GPS_LM_FIX_REQUEST_CONTINUE', # 0x50d
            1294: 'EVENT_GPS_LM_PRM_REQUEST_CONTINUE', # 0x50e
            1295: 'EVENT_GPS_LM_PPM_REQUEST_CONTINUE', # 0x50f
            1296: 'EVENT_GPS_LM_AIDING_DATA_RECEIVED', # 0x510
            1297: 'EVENT_GPS_LM_RC_ON_TIMER_TIMEOUT', # 0x511
            1298: 'EVENT_GPS_LM_SHUT_OFF_TIMER_TIMEOUT', # 0x512
            1299: 'EVENT_GPS_LM_MGP_ON', # 0x513
            1300: 'EVENT_GPS_LM_MGP_IDLE', # 0x514
            1301: 'EVENT_GPS_LM_MGP_OFF', # 0x515
            1302: 'EVENT_DRM_RO_CONSUMPTION_VALIDATION', # 0x516
            1303: 'EVENT_DRM_RO_INSTALLATION_VALIDATION', # 0x517
            1304: 'EVENT_FLUTE_FDT_INST_RCVD', # 0x518
            1305: 'EVENT_FLUTE_FDT_INST_RCV_FAIL', # 0x519
            1306: 'EVENT_FLUTE_FDT_INST_EXPIRED', # 0x51a
            1307: 'EVENT_FLUTE_JOIN_SESSION_REQ_RCVD', # 0x51b
            1308: 'EVENT_FLUTE_LEAVE_SESSION_REQ_RCVD', # 0x51c
            1309: 'EVENT_FLUTE_SESSION_CLOSED', # 0x51d
            1310: 'EVENT_FLUTE_SESSION_CLOSED_BY_APP', # 0x51e
            1311: 'EVENT_FLUTE_B_FLAG_RCVD', # 0x51f
            1312: 'EVENT_FLUTE_GET_FILE_REQUEST_RCVD', # 0x520
            1313: 'EVENT_FLUTE_JOIN_SESSION_RSP', # 0x521
            1314: 'EVENT_FLUTE_FILE_STATUS_RSP', # 0x522
            1315: 'EVENT_FLUTE_CANCEL_FILE_REQ_RCVD', # 0x523
            1316: 'EVENT_DTV_DVBH_DEINIT_REQ_RCVD', # 0x524
            1317: 'EVENT_DTV_DVBH_DEINIT_SUCCESS', # 0x525
            1318: 'EVENT_DTV_DVBH_DEINIT_FAILURE', # 0x526
            1319: 'EVENT_CONTENT_INSTALL_BEGIN', # 0x527
            1320: 'EVENT_CONTENT_INSTALL_COMPLETE', # 0x528
            1321: 'EVENT_CONTENT_RETRIEVAL_BEGIN', # 0x529
            1322: 'EVENT_CONTENT_RETRIEVAL_COMPLETE', # 0x52a
            1323: 'EVENT_CONTENT_BACKUP_BEGIN', # 0x52b
            1324: 'EVENT_CONTENT_BACKUP_COMPLETE', # 0x52c
            1325: 'EVENT_CONTENT_FWD_BEGIN', # 0x52d
            1326: 'EVENT_CONTENT_FWD_COMPLETE', # 0x52e
            1327: 'EVENT_HARD_HANDOFF_VOIP_TO_CDMA', # 0x52f
            1328: 'EVENT_EAP_SIM_AKA', # 0x530
            1329: 'EVENT_WLAN_CP_MEAS', # 0x531
            1330: 'EVENT_WLAN_CP_HO', # 0x532
            1331: 'EVENT_WLAN_CP_11D', # 0x533
            1332: 'EVENT_WLAN_MC', # 0x534
            1333: 'EVENT_SVG_CONTENT_SET', # 0x535
            1334: 'EVENT_SVG_CONTENT_PLAY', # 0x536
            1335: 'EVENT_SVG_CONTENT_RESUME', # 0x537
            1336: 'EVENT_SVG_CONTENT_PAUSE', # 0x538
            1337: 'EVENT_SVG_CONTENT_STOP', # 0x539
            1338: 'EVENT_SVG_CONTENT_USEREVENT', # 0x53a
            1339: 'EVENT_SVG_CONTENT_GETURIDATA', # 0x53b
            1340: 'EVENT_SVG_CONTENT_TRANSFORM', # 0x53c
            1341: 'EVENT_SVG_GET_PARAM', # 0x53d
            1342: 'EVENT_SVG_SET_PARAM', # 0x53e
            1343: 'EVENT_WLAN_WPA2', # 0x53f
            1344: 'EVENT_WCDMA_PSC_SCANNER_STOP', # 0x540
            1345: 'EVENT_MEDIA_PLAYER_KEYPRESS', # 0x541
            1346: 'EVENT_WLAN_MC_QOS', # 0x542
            1347: 'EVENT_WCDMA_PSC_SCANNER_STATE', # 0x543
            1348: 'EVENT_WLAN_CP_ADHOC', # 0x544
            1349: 'EVENT_DMB_STACK_SHUTDOWN', # 0x545
            1350: 'EVENT_DMB_TUNE_DONE_SUCCESS', # 0x546
            1351: 'EVENT_DMB_TUNE_DONE_FAILURE', # 0x547
            1352: 'EVENT_DMB_SEARCH_DONE', # 0x548
            1353: 'EVENT_DMB_SCAN_DONE', # 0x549
            1354: 'EVENT_DMB_RECEPTION_INFO_CHANGED', # 0x54a
            1355: 'EVENT_DMB_DMB_GUIDE_CHANGED', # 0x54b
            1356: 'EVENT_DMB_LOCATION_INFO_CHANGED', # 0x54c
            1357: 'EVENT_DMB_LOST_ENSEMBLE', # 0x54d
            1358: 'EVENT_DMB_STREAM_TERMINATED', # 0x54e
            1359: 'EVENT_DMB_STREAM_DATA_AVAILABLE', # 0x54f
            1360: 'EVENT_DMB_RESERVED1', # 0x550
            1361: 'EVENT_DMB_RESERVED2', # 0x551
            1362: 'EVENT_DMB_RESERVED3', # 0x552
            1363: 'EVENT_DMB_RESERVED4', # 0x553
            1364: 'EVENT_DMB_RESERVED5', # 0x554
            1365: 'EVENT_DMB_RESERVED6', # 0x555
            1366: 'EVENT_DMB_RESERVED7', # 0x556
            1367: 'EVENT_DMB_RESERVED8', # 0x557
            1368: 'EVENT_DMB_RESERVED9', # 0x558
            1369: 'EVENT_DMB_RESERVED10', # 0x559
            1370: 'EVENT_MOBILEVIEW_RESERVED1', # 0x55a
            1371: 'EVENT_MOBILEVIEW_RESERVED2', # 0x55b
            1372: 'EVENT_MOBILEVIEW_RESERVED3', # 0x55c
            1373: 'EVENT_MOBILEVIEW_RESERVED4', # 0x55d
            1374: 'EVENT_MOBILEVIEW_RESERVED5', # 0x55e
            1375: 'EVENT_HDR_DOS_MO_DOS_STATUS', # 0x55f
            1376: 'EVENT_GPSONEXTRA_START_DOWNLOAD', # 0x560
            1377: 'EVENT_GPSONEXTRA_END_DOWNLOAD', # 0x561
            1378: 'EVENT_SNSD_GENERIC', # 0x562
            1379: 'EVENT_SNSD_DEVICE_INIT', # 0x563
            1380: 'EVENT_SNSD_DEVICE_CONFIGURED', # 0x564
            1381: 'EVENT_SNSD_EVENT_DATA_READY', # 0x565
            1382: 'EVENT_SNSD_EVENT_COND_MET', # 0x566
            1383: 'EVENT_SNSD_DEVICE_DOWN', # 0x567
            1384: 'EVENT_SNSD_ERROR', # 0x568
            1385: 'EVENT_CM_COUNTRY_SELECTED', # 0x569
            1386: 'EVENT_CM_SELECT_COUNTRY', # 0x56a
            1387: 'EVENT_GPS_DCME_NEW_SV_ADDED_IN_AA', # 0x56b
            1388: 'EVENT_GPS_DCME_SV_REMOVED_FROM_AA', # 0x56c
            1389: 'EVENT_ESG_GET_PROV_LIST_REQ_RCVD', # 0x56d
            1390: 'EVENT_ESG_GET_PROV_LIST_REQ_FAIL', # 0x56e
            1391: 'EVENT_ESG_PROV_LIST_AVAILABLE', # 0x56f
            1392: 'EVENT_ESG_ACQ_REQ_RCVD', # 0x570
            1393: 'EVENT_ESG_ACQ_REQ_FAIL', # 0x571
            1394: 'EVENT_ESG_STOP_REQ_RCVD', # 0x572
            1395: 'EVENT_ESG_STOP_REQ_FAIL', # 0x573
            1396: 'EVENT_ESG_STOP_COMPLETE', # 0x574
            1397: 'EVENT_ADC_ONDIE_THERM_READ', # 0x575
            1398: 'EVENT_CONTENT_NO_VALID_OR_EXPIRED_RIGHTS', # 0x576
            1399: 'EVENT_MOBILEVIEW_RESERVED30', # 0x577
            1400: 'EVENT_MOBILEVIEW_RESERVED31', # 0x578
            1401: 'EVENT_MOBILEVIEW_RESERVED32', # 0x579
            1402: 'EVENT_GPS_DCME_MEAS_CYCLE_START', # 0x57a
            1403: 'EVENT_GPS_DCME_MEAS_CYCLE_END', # 0x57b
            1404: 'EVENT_GPS_CME_ENGAGED', # 0x57c
            1405: 'EVENT_GPS_CME_NOT_ENGAGED', # 0x57d
            1406: 'EVENT_GPS_DCME_ENGAGED', # 0x57e
            1407: 'EVENT_GPS_DCME_NOT_ENGAGED', # 0x57f
            1408: 'EVENT_HS_USB_DEVICE_ATTACHED', # 0x580
            1409: 'EVENT_HS_USB_HID_DISCONECT', # 0x581
            1410: 'EVENT_HS_USB_HID_CONNECT', # 0x582
            1411: 'EVENT_HS_USB_MSD_CONNECT', # 0x583
            1412: 'EVENT_HS_USB_MSD_DISCONECT', # 0x584
            1413: 'EVENT_HS_USB_STACK_SUSPENDED', # 0x585
            1414: 'EVENT_HS_USB_STACK_RESUMED', # 0x586
            1415: 'EVENT_HS_USB_ENTER_HOST_MODE', # 0x587
            1416: 'EVENT_HS_USB_OPERATIONAL_ERROR', # 0x588
            1417: 'EVENT_DTV_L1_ACQ_DONE', # 0x589
            1418: 'EVENT_DTV_L1_SCAN', # 0x58a
            1419: 'EVENT_DTV_L1_ONLINE', # 0x58b
            1420: 'EVENT_DTV_L1_SNOOZE', # 0x58c
            1421: 'EVENT_DTV_L1_SLEEP', # 0x58d
            1422: 'EVENT_DTV_L1_HANDOFF', # 0x58e
            1423: 'EVENT_DTV_L1_SIGNAL_LOST', # 0x58f
            1424: 'EVENT_IMS_SIP_REGISTRATION_START', # 0x590
            1425: 'EVENT_IMS_SIP_REGISTER_END', # 0x591
            1426: 'EVENT_IMS_SIP_DEREGISTER_START', # 0x592
            1427: 'EVENT_IMS_SIP_DEREGISTER_END', # 0x593
            1428: 'EVENT_IMS_SIP_SESSION_START', # 0x594
            1429: 'EVENT_IMS_SIP_SESSION_RINGING', # 0x595
            1430: 'EVENT_IMS_SIP_SESSION_ESTABLISHED', # 0x596
            1431: 'EVENT_IMS_SIP_SESSION_TERMINATED', # 0x597
            1432: 'EVENT_IMS_SIP_SESSION_CANCEL', # 0x598
            1433: 'EVENT_IMS_SIP_SESSION_FAILURE', # 0x599
            1434: 'EVENT_IMS_SIP_RESPONSE_RECV', # 0x59a
            1435: 'EVENT_IMS_SIP_REQUEST_RECV', # 0x59b
            1436: 'EVENT_IMS_SIP_RESPONSE_SEND', # 0x59c
            1437: 'EVENT_IMS_SIP_REQUEST_SEND', # 0x59d
            1438: 'EVENT_WLAN_TKIP_COUNTER_MEAS', # 0x59e
            1439: 'EVENT_GPS_BLANKING_OFF', # 0x59f
            1440: 'EVENT_GPS_BLANKING_ON', # 0x5a0
            1441: 'EVENT_MMGSDI_EVENT', # 0x5a1
            1442: 'EVENT_WLAN_CP_SYS_MGR_STATE_TRANS', # 0x5a2
            1443: 'EVENT_GPS_OPTIMISTIC_PUNC_START', # 0x5a3
            1444: 'EVENT_GPS_OPTIMISTIC_PUNC_END', # 0x5a4
            1445: 'EVENT_QVP_SEND_RTP_PACKET', # 0x5a5
            1446: 'EVENT_QVP_RECV_RTP_PACKET', # 0x5a6
            1447: 'EVENT_HDR_IDLE_REACQ_FAIL_DDARF', # 0x5a7
            1448: 'EVENT_BCAST_SEC_STKM_PARSE_STATUS', # 0x5a8
            1449: 'EVENT_BCAST_SEC_STKM_RECEIVED', # 0x5a9
            1450: 'EVENT_BCAST_SEC_SDP_PARSE_STATUS', # 0x5aa
            1451: 'EVENT_CGPS_ME_DPO_STATUS', # 0x5ab
            1452: 'EVENT_GPS_SV_SEARCH_STATE', # 0x5ac
            1453: 'EVENT_GPS_TM_ON_DEMAND_MODE_CHANGE', # 0x5ad
            1454: 'EVENT_GPS_TM_ON_DEMAND_BEGIN', # 0x5ae
            1455: 'EVENT_GPS_TM_ON_DEMAND_DONE', # 0x5af
            1456: 'EVENT_RMAC_CARRIER_STATE_CHANGED', # 0x5b0
            1457: 'EVENT_GPS_SBAS_DEMOD_REPORT', # 0x5b1
            1458: 'EVENT_GPS_EXTERN_COARSE_POS_INJ_START', # 0x5b2
            1459: 'EVENT_GPS_EXTERN_COARSE_POS_INJ_END', # 0x5b3
            1460: 'EVENT_GPS_EPH_REREQUEST_TIME', # 0x5b4
            1461: 'EVENT_WLAN_QOS_PSTREAM', # 0x5b5
            1462: 'EVENT_WLAN_CP_VCC', # 0x5b6
            1463: 'EVENT_CGPS_DIAG_FIRST_SUCCESSFUL_FIX', # 0x5b7
            1464: 'EVENT_EUL_RECONFIG_OR_ASU_OR_TTI_RECFG', # 0x5b8
            1465: 'EVENT_DS707_PKT_LN_UPDATE', # 0x5b9
            1466: 'EVENT_DS707_PKT_IDM_CHANGE', # 0x5ba
            1467: 'EVENT_RLP_QN_ADD', # 0x5bb
            1468: 'EVENT_RLP_QN_DROP', # 0x5bc
            1469: 'EVENT_RLP_MULTILINK_NAK', # 0x5bd
            1470: 'EVENT_RLP_REV_LINK_NAK', # 0x5be
            1471: 'EVENT_GSTK_EVENT', # 0x5bf
            1472: 'EVENT_GAN_REGISTRATION_REQUEST', # 0x5c0
            1473: 'EVENT_GAN_REGISTER_ACCEPT', # 0x5c1
            1474: 'EVENT_CALL_RINGING_ALERT', # 0x5c2
            1475: 'EVENT_GAN_PAGING_RECEIVED', # 0x5c3
            1476: 'EVENT_GAN_CALL_DISCONNECT', # 0x5c4
            1477: 'EVENT_GAN_CALL_RELEASE_COMPLETE', # 0x5c5
            1478: 'EVENT_GAN_HANDIN_COMMAND', # 0x5c6
            1479: 'EVENT_GAN_HANDIN_COMPLETE', # 0x5c7
            1480: 'EVENT_GAN_HANDOUT_COMMAND', # 0x5c8
            1481: 'EVENT_GAN_HANDOUT_COMPLETE', # 0x5c9
            1482: 'EVENT_GAN_SMS_START', # 0x5ca
            1483: 'EVENT_GAN_SMS_ACK', # 0x5cb
            1484: 'EVENT_GAN_QDJ_ENQUEUE', # 0x5cc
            1485: 'EVENT_GAN_QDJ_DEQUEUE', # 0x5cd
            1486: 'EVENT_GAN_ACTIVATE_DATA_CHANNEL', # 0x5ce
            1487: 'EVENT_GAN_DATA_CHANNEL_CONNECTED', # 0x5cf
            1488: 'EVENT_GAN_RLP_SUSPEND', # 0x5d0
            1489: 'EVENT_GAN_RLP_RESUME', # 0x5d1
            1490: 'EVENT_GAN_WAKEUP_REQ', # 0x5d2
            1491: 'EVENT_GAN_WAKEUP_CNF', # 0x5d3
            1492: 'EVENT_GAN_HIBERNATION_REQ', # 0x5d4
            1493: 'EVENT_GAN_HIBERNATION_CNF', # 0x5d5
            1494: 'EVENT_WCDMA_UL_AMR_RATE', # 0x5d6
            1495: 'EVENT_EUL_TTI_RECONFIG', # 0x5d7
            1496: 'EVENT_WCDMA_CONN_REL_CAUSE', # 0x5d8
            1497: 'EVENT_WCDMA_CONN_REQ_CAUSE', # 0x5d9
            1498: 'EVENT_LTE_TIMING_ADVANCE', # 0x5da
            1499: 'EVENT_LTE_UL_OUT_OF_SYNC', # 0x5db
            1500: 'EVENT_LTE_SPS_DEACTIVATED', # 0x5dc
            1501: 'EVENT_LTE_RACH_ACCESS_START', # 0x5dd
            1502: 'EVENT_LTE_RACH_RAID_MATCH', # 0x5de
            1503: 'EVENT_LTE_RACH_ACCESS_RESULT', # 0x5df
            1504: 'EVENT_DTV_L1_POWERUP', # 0x5e0
            1505: 'EVENT_DTV_L1_POWERDOWN', # 0x5e1
            1506: 'EVENT_DTV_L1_SOFT_RESET', # 0x5e2
            1507: 'EVENT_DTV_L1_STATE_CHANGE', # 0x5e3
            1508: 'EVENT_DTV_L1_ACQ_TUNE_STATUS', # 0x5e4
            1509: 'EVENT_DTV_L1_ACQ_DONE_STATUS', # 0x5e5
            1510: 'EVENT_DTV_L1_ACQ_FAIL', # 0x5e6
            1511: 'EVENT_DTV_L1_TRAFFIC_STARTED', # 0x5e7
            1512: 'EVENT_DTV_L1_BAD_FRAME_RECEIVED', # 0x5e8
            1513: 'EVENT_DTV_L1_TMCC_FAILURE', # 0x5e9
            1514: 'EVENT_DTV_L1_RECOVERY_STATUS', # 0x5ea
            1515: 'EVENT_DTV_L1_INTERRUPT_LOG_RECEIVED', # 0x5eb
            1516: 'EVENT_DTV_L1_L3_API_COMMAND', # 0x5ec
            1517: 'EVENT_DTV_L1_MODEM_FAILURE', # 0x5ed
            1518: 'EVENT_GSM_CALL_DROP', # 0x5ee
            1519: 'EVENT_GSM_ACCESS_FAILURE', # 0x5ef
            1520: 'EVENT_DTV_ISDB_ACTIVATE', # 0x5f0
            1521: 'EVENT_DTV_ISDB_DEACTIVATE', # 0x5f1
            1522: 'EVENT_DTV_ISDB_TUNE', # 0x5f2
            1523: 'EVENT_DTV_ISDB_UNTUNE', # 0x5f3
            1524: 'EVENT_DTV_ISDB_SELECT_SERVICE', # 0x5f4
            1525: 'EVENT_DTV_ISDB_SERVICE_AVAILABLE', # 0x5f5
            1526: 'EVENT_DTV_ISDB_TRAFFIC_LOST', # 0x5f6
            1527: 'EVENT_DTV_ISDB_TABLE_UPDATE', # 0x5f7
            1528: 'EVENT_DTV_ISDB_TRACKS_SELECTED', # 0x5f8
            1529: 'EVENT_DTV_ISDB_PES_BUFFER_OVERFLOW', # 0x5f9
            1530: 'EVENT_DTV_ISDB_PES_BUFFER_UNDERFLOW', # 0x5fa
            1531: 'EVENT_DTV_ISDB_ACQUIRE_DATA_COMPONENT', # 0x5fb
            1532: 'EVENT_DTV_ISDB_STOP_COMPONENT_ACQUISITION', # 0x5fc
            1533: 'EVENT_DTV_ISDB_DII_CHANGED', # 0x5fd
            1534: 'EVENT_DTV_ISDB_DATA_EVENT_MESSAGE', # 0x5fe
            1535: 'EVENT_DTV_ISDB_MODULE_CONSTRUCTION', # 0x5ff
            1536: 'EVENT_DTV_ISDB_PARSING_ERROR', # 0x600
            1537: 'EVENT_HDR_SLP_SLPQH_TIMER_STARTED', # 0x601
            1538: 'EVENT_HDR_SLP_SLPQH_TIMER_STOPPED', # 0x602
            1539: 'EVENT_HDR_SLP_SLPQH_NUM_PENDING_MSGS', # 0x603
            1540: 'EVENT_HDR_OVHD_FIND_CACHED_MSG', # 0x604
            1541: 'EVENT_WCDMA_RRC_TIMER_EXPIRED', # 0x605
            1542: 'EVENT_WCDMA_UOOS_TIMER_USED', # 0x606
            1543: 'EVENT_WCDMA_UOOS_TIMER_START', # 0x607
            1544: 'EVENT_WCDMA_UOOS_TIMER_STOP', # 0x608
            1545: 'EVENT_WCDMA_UOOS_TIME_REMAINING', # 0x609
            1546: 'EVENT_WCDMA_RRCCSP_SCAN_START', # 0x60a
            1547: 'EVENT_WCDMA_ACQUISITON_SUCCESS', # 0x60b
            1548: 'EVENT_WCDMA_CELL_SELECTION_FAIL', # 0x60c
            1549: 'EVENT_WCDMA_BPLMN_START', # 0x60d
            1550: 'EVENT_WCDMA_BPLMN_END', # 0x60e
            1551: 'EVENT_WCDMA_BPLMN_SCAN_START', # 0x60f
            1552: 'EVENT_WCDMA_BPLMN_SCAN_END', # 0x610
            1553: 'EVENT_MSG_HIGH', # 0x611
            1554: 'EVENT_MSG_MED', # 0x612
            1555: 'EVENT_MSG_LOW', # 0x613
            1556: 'EVENT_MSG_ERROR', # 0x614
            1557: 'EVENT_MSG_FATAL', # 0x615
            1558: 'EVENT_GAN_START_TU3910', # 0x616
            1559: 'EVENT_GAN_STOP_TU3910', # 0x617
            1560: 'EVENT_GAN_EXPIRY_TU3910', # 0x618
            1561: 'EVENT_GAN_START_TU3920', # 0x619
            1562: 'EVENT_GAN_STOP_TU3920', # 0x61a
            1563: 'EVENT_GAN_EXPIRY_TU3920', # 0x61b
            1564: 'EVENT_GAN_START_TU3906', # 0x61c
            1565: 'EVENT_GAN_STOP_TU3906', # 0x61d
            1566: 'EVENT_GAN_EXPIRY_TU3906', # 0x61e
            1567: 'EVENT_GAN_URR_REGISTER_UPDATE', # 0x61f
            1568: 'EVENT_IPSEC_IKE_SA_INIT_SENT', # 0x620
            1569: 'EVENT_IPSEC_IKE_SA_INIT_RECV', # 0x621
            1570: 'EVENT_IPSEC_IKE_SA_ESTABLISHED', # 0x622
            1571: 'EVENT_IPSEC_IKE_AUTH_SENT', # 0x623
            1572: 'EVENT_IPSEC_IKE_AUTH_RECV', # 0x624
            1573: 'EVENT_IPSEC_IKE_EAP_START', # 0x625
            1574: 'EVENT_IPSEC_IKE_EAP_FINISH', # 0x626
            1575: 'EVENT_IPSEC_CHILD_SA_ESTABLISHED', # 0x627
            1576: 'EVENT_IPSEC_IKE_INFO_MSG_SENT', # 0x628
            1577: 'EVENT_IPSEC_IKE_INFO_MSG_RECV', # 0x629
            1578: 'EVENT_IPSEC_CREATE_CHILD_SA_SENT', # 0x62a
            1579: 'EVENT_IPSEC_CREATE_CHILD_SA_RECV', # 0x62b
            1580: 'EVENT_IPSEC_IKE_SA_DELETE_START', # 0x62c
            1581: 'EVENT_IPSEC_IKE_SA_DELETE_DONE', # 0x62d
            1582: 'EVENT_IPSEC_CHILD_SA_DELETE_START', # 0x62e
            1583: 'EVENT_IPSEC_CHILD_SA_DELETE_DONE', # 0x62f
            1584: 'EVENT_IPSEC_IKE_SA_REKEY_START', # 0x630
            1585: 'EVENT_IPSEC_IKE_SA_REKEY_DONE', # 0x631
            1586: 'EVENT_IPSEC_CHILD_SA_REKEY_START', # 0x632
            1587: 'EVENT_IPSEC_CHILD_SA_REKEY_DONE', # 0x633
            1588: 'EVENT_IPSEC_IKE_MESG_RETRANSMIT', # 0x634
            1589: 'EVENT_IPSEC_IKE_NAT_DETECTED', # 0x635
            1590: 'EVENT_IPSEC_IKE_NAT_KEEPALIVE_SENT', # 0x636
            1591: 'EVENT_IPSEC_IKE_DPD_SENT', # 0x637
            1592: 'EVENT_IPSEC_IKE_ERR_NOTIFY_SENT', # 0x638
            1593: 'EVENT_IPSEC_IKE_ERR_NOTIFY_RECV', # 0x639
            1594: 'EVENT_GAN_ROVEIN_CNF', # 0x63a
            1595: 'EVENT_GAN_ROVEOUT_CNF', # 0x63b
            1596: 'EVENT_GAN_RRC_ROVEIN_CNF', # 0x63c
            1597: 'EVENT_GAN_RRC_ROVEIN_REJ', # 0x63d
            1598: 'EVENT_GAN_RRC_ROVEOUT_CNF', # 0x63e
            1599: 'EVENT_GAN_RRC_ROVEOUT_REJ', # 0x63f
            1600: 'EVENT_GPSXTRA_T_SESS_BEGIN', # 0x640
            1601: 'EVENT_GPSXTRA_T_SESS_DATA', # 0x641
            1602: 'EVENT_GPSXTRA_T_SESS_DONE', # 0x642
            1603: 'EVENT_GPSXTRA_T_SESS_END', # 0x643
            1604: 'EVENT_DS_GO_NULL_TIMER', # 0x644
            1607: 'EVENT_LTE_RRC_OUT_OF_SERVICE', # 0x647
            1608: 'EVENT_LTE_RRC_RADIO_LINK_FAILURE', # 0x648
            1611: 'EVENT_LTE_RRC_NEW_CELL_IND', # 0x64b
            1612: 'EVENT_LTE_RRC_CELL_RESEL_FAILURE', # 0x64c
            1613: 'EVENT_LTE_RRC_HO_FAILURE', # 0x64d
            1615: 'EVENT_LTE_RRC_IRAT_HO_FROM_EUTRAN', # 0x64f
            1616: 'EVENT_LTE_RRC_IRAT_HO_FROM_EUTRAN_FAILURE', # 0x650
            1617: 'EVENT_LTE_RRC_IRAT_RESEL_FROM_EUTRAN', # 0x651
            1618: 'EVENT_LTE_RRC_IRAT_RESEL_FROM_EUTRAN_FAILURE', # 0x652
            1619: 'EVENT_LTE_RRC_SIB_READ_FAILURE', # 0x653
            1620: 'EVENT_GAN_ROVEIN_REQ', # 0x654
            1621: 'EVENT_GAN_ROVEOUT_REQ', # 0x655
            1622: 'EVENT_MBP_RF_ANALOG_JD_MODE_CHANGE', # 0x656
            1623: 'EVENT_MBP_RF_ANALOG_JD_INT', # 0x657
            1624: 'EVENT_CGPS_QWIP_SYSD_TRANSITION', # 0x658
            1625: 'EVENT_HPLMN_TIMER_EXPIRED', # 0x659
            1626: 'EVENT_GSDI_GET_FEATURE_INDICATOR_DATA', # 0x65a
            1639: 'EVENT_SNS_CONTEXT_OPEN', # 0x667
            1640: 'EVENT_SNS_CONTEXT_CLOSE', # 0x668
            1641: 'EVENT_SNS_COND_SET', # 0x669
            1642: 'EVENT_SNS_COND_CANCEL', # 0x66a
            1643: 'EVENT_SNS_COND_MET', # 0x66b
            1644: 'EVENT_SNS_DATA_START', # 0x66c
            1645: 'EVENT_SNS_DATA_STOP', # 0x66d
            1646: 'EVENT_WCDMA_RLC_CONFIG', # 0x66e
            1647: 'EVENT_HSPA_PLUS_CFG', # 0x66f
            1648: 'EVENT_SNS_DRIVER_STATE_CHANGE', # 0x670
            1649: 'EVENT_WCDMA_TIMER_DISCARD_EXPIRY', # 0x671
            1650: 'EVENT_NAS_CB_PAGE_RECEIVED', # 0x672
            1651: 'EVENT_WCDMA_RLC_RESET', # 0x673
            1652: 'EVENT_HDR_MRLP_EHRPD_PERSONALITY_IS_ACTIVE', # 0x674
            1653: 'EVENT_WLAN_SECURITY', # 0x675
            1654: 'EVENT_WLAN_STATUS', # 0x676
            1655: 'EVENT_WLAN_HANDOFF', # 0x677
            1656: 'EVENT_WLAN_VCC', # 0x678
            1657: 'EVENT_WLAN_QOS', # 0x679
            1658: 'EVENT_WLAN_PE', # 0x67a
            1659: 'EVENT_WLAN_ADD_BLOCK_ACK_SUCCESS', # 0x67b
            1660: 'EVENT_WLAN_ADD_BLOCK_ACK_FAILED', # 0x67c
            1661: 'EVENT_WLAN_DELETE_BLOCK_ACK_SUCCESS', # 0x67d
            1662: 'EVENT_WLAN_DELETE_BLOCK_ACK_FAILED', # 0x67e
            1663: 'EVENT_WLAN_BSS_PROTECTION', # 0x67f
            1664: 'EVENT_WLAN_BRINGUP_STATUS', # 0x680
            1665: 'EVENT_WLAN_POWERSAVE_GENERIC', # 0x681
            1666: 'EVENT_WLAN_POWERSAVE_WOW', # 0x682
            1667: 'EVENT_WLAN_WCM', # 0x683
            1668: 'EVENT_WLAN_WPS_SCAN_START', # 0x684
            1669: 'EVENT_WLAN_WPS_SCAN_COMPLETE', # 0x685
            1670: 'EVENT_WLAN_WPS_CONNECT_REQUEST', # 0x686
            1671: 'EVENT_WLAN_WPS_CONNECT_RESPONSE', # 0x687
            1672: 'EVENT_WLAN_WPS_PBC_SESSION_OVERLAP', # 0x688
            1673: 'EVENT_WLAN_WPS_PBC_WALK_TIMER_START', # 0x689
            1674: 'EVENT_WLAN_WPS_PBC_WALK_TIMER_STOP', # 0x68a
            1675: 'EVENT_WLAN_WPS_PBC_AP_DETECTED', # 0x68b
            1676: 'EVENT_WLAN_WPS_REGISTRATION_START', # 0x68c
            1677: 'EVENT_WLAN_WPS_WSC_MESSAGE', # 0x68d
            1678: 'EVENT_WLAN_WPS_DISCOVERY', # 0x68e
            1679: 'EVENT_WLAN_WPS_REGISTRATION_COMPLETE', # 0x68f
            1680: 'EVENT_WLAN_WPS_DISCONNECT', # 0x690
            1681: 'EVENT_WLAN_BTC', # 0x691
            1683: 'EVENT_IPV6_SM_TRANSITION', # 0x693
            1685: 'EVENT_LTE_ML1_STATE_CHANGE', # 0x695
            1686: 'EVENT_AUTH_PROTO', # 0x696
            1687: 'EVENT_VSNCP', # 0x697
            1688: 'EVENT_IID', # 0x698
            1689: 'EVENT_IMS_VIDEOSHARE_REGISTRATION_SUCCESS', # 0x699
            1690: 'EVENT_IMS_VIDEOSHARE_INVITE_SENT', # 0x69a
            1691: 'EVENT_IMS_VIDEOSHARE_INCOMING_INVITE', # 0x69b
            1692: 'EVENT_IMS_VIDEOSHARE_ACCEPT_REJECT_INVITE', # 0x69c
            1693: 'EVENT_IMS_VIDEOSHARE_ACCEPTING_SESSION', # 0x69d
            1694: 'EVENT_IMS_VIDEOSHARE_SESSION_ESTABLISHED', # 0x69e
            1695: 'EVENT_IMS_VIDEOSHARE_END_SESSION', # 0x69f
            1696: 'EVENT_IMS_VIDEOSHARE_PREVIEW_VIDEO_FRAME', # 0x6a0
            1697: 'EVENT_IMS_VIDEOSHARE_DECODED_VIDEO_FRAME', # 0x6a1
            1698: 'EVENT_IMS_VIDEOSHARE_RECEIVING', # 0x6a2
            1699: 'EVENT_IMS_VIDEOSHARE_START_APPLICATION', # 0x6a3
            1700: 'EVENT_IMS_VIDEOSHARE_END_APPLICATION', # 0x6a4
            1701: 'EVENT_IMS_VIDEOSHARE_CAPABILITY_SUCCESS', # 0x6a5
            1702: 'EVENT_IMS_VIDEOSHARE_MEDIA_RECORDING_RESOURCE_ACQUIRED', # 0x6a6
            1703: 'EVENT_IMS_VIDEOSHARE_MEDIA_RECORDING_RESOURCE_RELEASED', # 0x6a7
            1704: 'EVENT_IMS_VIDEOSHARE_SENDING', # 0x6a8
            1705: 'EVENT_IMS_VIDEOSHARE_INCOMING_OPTION_RECEIVED', # 0x6a9
            1706: 'EVENT_IMS_VIDEOSHARE_INCOMING_OPTION_RESPONDED', # 0x6aa
            1707: 'EVENT_IMS_VIDEOSHARE_ERR_CALL_FAILED', # 0x6ab
            1708: 'EVENT_IMS_VIDEOSHARE_ERR_REGISTRATION_FAILED', # 0x6ac
            1709: 'EVENT_IMS_VIDEOSHARE_ERR_RECORDER_ERROR', # 0x6ad
            1710: 'EVENT_IMS_VIDEOSHARE_ERR_PLAYER_ERROR', # 0x6ae
            1711: 'EVENT_IMS_VIDEOSHARE_ERR_MEDIA_SESSION_FAILURE', # 0x6af
            1712: 'EVENT_IMS_VIDEOSHARE_ERR_CAPABILITY_FAILURE', # 0x6b0
            1713: 'EVENT_IMS_VIDEOSHARE_ERR_MEDIA_RECORDING_FAILED', # 0x6b1
            1714: 'EVENT_WLAN_PE_FRAME', # 0x6b2
            1715: 'EVENT_SNS_VCPS_HEADING_COMPUTED', # 0x6b3
            1716: 'EVENT_SNS_VCPS_TRACKED_CAL_SET_SAVED', # 0x6b4
            1717: 'EVENT_GNSS_PRESC_DWELL_COMPLETE', # 0x6b5
            1718: 'EVENT_LTE_MAC_RESET', # 0x6b6
            1719: 'EVENT_LTE_BSR_SR_REQUEST', # 0x6b7
            1720: 'EVENT_LTE_MAC_TIMER', # 0x6b8
            1721: 'EVENT_CM_DS_OPERATIONAL_MODE', # 0x6b9
            1722: 'EVENT_CM_DS_MODE_PREF', # 0x6ba
            1723: 'EVENT_CM_DS_GW_ACQ_ORDER_PREF', # 0x6bb
            1724: 'EVENT_CM_DS_SRV_DOMAIN_PREF', # 0x6bc
            1725: 'EVENT_CM_DS_BAND_PREF', # 0x6bd
            1726: 'EVENT_CM_DS_ROAM_PREF', # 0x6be
            1727: 'EVENT_CM_DS_HYBRID_PREF', # 0x6bf
            1728: 'EVENT_CM_DS_NETWORK_SEL_MODE_PREF', # 0x6c0
            1729: 'EVENT_CM_DS_CALL_EVENT_ORIG', # 0x6c1
            1730: 'EVENT_CM_DS_CALL_EVENT_CONNECT', # 0x6c2
            1731: 'EVENT_CM_DS_CALL_EVENT_END', # 0x6c3
            1732: 'EVENT_CM_DS_ENTER_EMERGENCY_CB', # 0x6c4
            1733: 'EVENT_CM_DS_EXIT_EMERGENCY_CB', # 0x6c5
            1734: 'EVENT_CM_DS_CALL_STATE', # 0x6c6
            1735: 'EVENT_CM_DS_DS_INTERRAT_STATE', # 0x6c7
            1736: 'EVENT_CM_DS_CELL_SRV_IND', # 0x6c8
            1737: 'EVENT_CM_DS_COUNTRY_SELECTED', # 0x6c9
            1738: 'EVENT_CM_DS_DATA_AVAILABLE', # 0x6ca
            1739: 'EVENT_CM_DS_SELECT_COUNTRY', # 0x6cb
            1740: 'EVENT_CM_DS_CALL_EVENT_ORIG_THR', # 0x6cc
            1741: 'EVENT_CM_DS_PLMN_FOUND', # 0x6cd
            1742: 'EVENT_CM_DS_SERVICE_CONFIRMED', # 0x6ce
            1743: 'EVENT_CM_DS_GET_PASSWORD_IND', # 0x6cf
            1744: 'EVENT_CM_DS_PASSWORD_AUTHENTICATION_STATUS', # 0x6d0
            1745: 'EVENT_CM_DS_USS_RESPONSE_NOTIFY_IND', # 0x6d1
            1746: 'EVENT_CM_DS_LCS_MOLR_CONF', # 0x6d2
            1747: 'EVENT_DS_NAS_MESSAGE_SENT', # 0x6d3
            1748: 'EVENT_DS_NAS_MESSAGE_RECEIVED', # 0x6d4
            1749: 'EVENT_DS_MM_STATE', # 0x6d5
            1750: 'EVENT_DS_GMM_STATE', # 0x6d6
            1751: 'EVENT_DS_PLMN_INFORMATION', # 0x6d7
            1752: 'EVENT_DIAG_STRESS_TEST_COMPLETED', # 0x6d8
            1753: 'EVENT_GNSS_CC_STATUS', # 0x6d9
            1754: 'EVENT_SNS_USER_STATE_CHANGE', # 0x6da
            1755: 'EVENT_DS_HPLMN_TIMER_EXPIRED', # 0x6db
            1756: 'EVENT_DS_RAT_CHANGE', # 0x6dc
            1757: 'EVENT_DTV_CMMB_API_CALL_ACTIVATE', # 0x6dd
            1758: 'EVENT_DTV_CMMB_API_CALL_DEACTIVATE', # 0x6de
            1759: 'EVENT_DTV_CMMB_API_CALL_TUNE', # 0x6df
            1760: 'EVENT_DTV_CMMB_API_CALL_SELECT_SERVICE', # 0x6e0
            1761: 'EVENT_DTV_CMMB_API_CALL_DESELECT_SERVICE', # 0x6e1
            1762: 'EVENT_DTV_CMMB_API_CALL_GET_SIGNAL_PARAMETERS', # 0x6e2
            1763: 'EVENT_DTV_CMMB_API_CALL_GET_NIT', # 0x6e3
            1764: 'EVENT_DTV_CMMB_API_CALL_GET_CMCT', # 0x6e4
            1765: 'EVENT_DTV_CMMB_API_CALL_GET_SMCT', # 0x6e5
            1766: 'EVENT_DTV_CMMB_API_CALL_GET_CSCT', # 0x6e6
            1767: 'EVENT_DTV_CMMB_API_CALL_GET_SSCT', # 0x6e7
            1768: 'EVENT_DTV_CMMB_API_CALL_GET_EADT', # 0x6e8
            1769: 'EVENT_DTV_CMMB_API_CALL_REQUEST_CA_CARD_NUMBER', # 0x6e9
            1770: 'EVENT_DTV_CMMB_API_CALL_REQUEST_CAS_ID', # 0x6ea
            1771: 'EVENT_DTV_CMMB_API_CALL_REGISTER_FOR_CONTROL_NOTIFICATIONS', # 0x6eb
            1772: 'EVENT_DTV_CMMB_API_CALL_DEREGISTER_FROM_CONTROL_NOTIFICATIONS', # 0x6ec
            1773: 'EVENT_DTV_CMMB_API_NOTIFICATION_ACTIVATE', # 0x6ed
            1774: 'EVENT_DTV_CMMB_API_NOTIFICATION_DEACTIVATE', # 0x6ee
            1775: 'EVENT_DTV_CMMB_API_NOTIFICATION_TUNE', # 0x6ef
            1776: 'EVENT_DTV_CMMB_API_NOTIFICATION_SELECT_SERVICE', # 0x6f0
            1777: 'EVENT_DTV_CMMB_API_NOTIFICATION_DESELECT_SERVICE', # 0x6f1
            1778: 'EVENT_DTV_CMMB_API_NOTIFICATION_TABLE_UPDATE', # 0x6f2
            1779: 'EVENT_DTV_CMMB_API_NOTIFICATION_SIGNAL_PARAMETERS', # 0x6f3
            1780: 'EVENT_DTV_CMMB_API_NOTIFICATION_AUTHORIZATION_FAILURE', # 0x6f4
            1781: 'EVENT_DTV_CMMB_API_NOTIFICATION_REGISTER_FOR_CONTROL_NOTIFICATIONS_COMPLETE', # 0x6f5
            1782: 'EVENT_DTV_CMMB_API_NOTIFICATION_DEREGISTER_FROM_CONTROL_NOTIFICATIONS_COMPLETE', # 0x6f6
            1783: 'EVENT_DTV_CMMB_API_NOTIFICATION_CA_CARD_NUMBER', # 0x6f7
            1784: 'EVENT_DTV_CMMB_API_NOTIFICATION_CAS_ID', # 0x6f8
            1785: 'EVENT_DTV_CMMB_API_NOTIFICATION_EMERGENCY_BROADCASTING_TRIGGER', # 0x6f9
            1786: 'EVENT_DTV_CMMB_API_NOTIFICATION_EMERGENCY_BROADCASTING_MESSAGE', # 0x6fa
            1787: 'EVENT_DTV_CMMB_API_CALL_REGISTER_FOR_ESG_NOTIFICATIONS', # 0x6fb
            1788: 'EVENT_DTV_CMMB_API_CALL_DEREGISTER_FROM_ESG_NOTIFICATIONS', # 0x6fc
            1789: 'EVENT_DTV_CMMB_API_CALL_GET_BASIC_DESCRIPTION_INFORMATION', # 0x6fd
            1790: 'EVENT_DTV_CMMB_API_CALL_SET_OUTPUT_PATH', # 0x6fe
            1791: 'EVENT_DTV_CMMB_API_NOTIFICATION_ESG_DATA_INFORMATION', # 0x6ff
            1792: 'EVENT_DTV_CMMB_API_NOTIFICATION_ESG_DATA_INFORMATION_DOWNLOAD_COMPLETE', # 0x700
            1793: 'EVENT_DTV_CMMB_API_NOTIFICATION_ESG_PROGRAM_INDICATION_INFORMATION', # 0x701
            1794: 'EVENT_DTV_CMMB_API_NOTIFICATION_REGISTER_FOR_ESG_NOTIFICATIONS_COMPLETE', # 0x702
            1795: 'EVENT_DTV_CMMB_API_NOTIFICATION_DEREGISTER_FROM_ESG_NOTIFICATIONS_COMPLETE', # 0x703
            1796: 'EVENT_DTV_CMMB_CAS_INITIALIZED', # 0x704
            1797: 'EVENT_DTV_CMMB_CAS_EMM_RECEIVED_AND_PROCESSED', # 0x705
            1798: 'EVENT_DTV_CMMB_CAS_ECM_RECEIVED_AND_PROCESSED', # 0x706
            1799: 'EVENT_ECALL_START', # 0x707
            1800: 'EVENT_ECALL_STOP', # 0x708
            1801: 'EVENT_ECALL_SESSION_START', # 0x709
            1802: 'EVENT_ECALL_SESSION_FAILURE', # 0x70a
            1803: 'EVENT_ECALL_SESSION_COMPLETE', # 0x70b
            1804: 'EVENT_ECALL_SESSION_RESET', # 0x70c
            1805: 'EVENT_ECALL_PSAP_MSD_DECODE_SUCCESS', # 0x70d
            1806: 'EVENT_ECALL_PSAP_LOST_SYNC', # 0x70e
            1807: 'EVENT_LTE_RRC_IRAT_REDIR_FROM_EUTRAN_START', # 0x70f
            1808: 'EVENT_LTE_RRC_IRAT_REDIR_FROM_EUTRAN_END', # 0x710
            1809: 'EVENT_GPRS_DS_CELL_CHANGE_ORDER', # 0x711
            1810: 'EVENT_GSM_DS_CELL_SELECTION_END', # 0x712
            1811: 'EVENT_GSM_DS_L1_STATE', # 0x713
            1812: 'EVENT_GSM_DS_PLMN_LIST_START', # 0x714
            1813: 'EVENT_GSM_DS_PLMN_LIST_END', # 0x715
            1814: 'EVENT_GSM_DS_POWER_SCAN_STATUS', # 0x716
            1815: 'EVENT_GSM_DS_RESELECT_START', # 0x717
            1816: 'EVENT_GSM_DS_RR_IN_SERVICE', # 0x718
            1817: 'EVENT_GSM_DS_RR_OUT_OF_SERVICE', # 0x719
            1818: 'EVENT_GSM_DS_TIMER_EXPIRED', # 0x71a
            1819: 'EVENT_GSM_DS_TO_WCDMA_RESELECT_END', # 0x71b
            1820: 'EVENT_CM_DS_SYSTEM_MODE', # 0x71c
            1821: 'EVENT_SD_DS_EVENT_ACTION', # 0x71d
            1822: 'EVENT_SMGMM_DS_REQUEST_SENT', # 0x71e
            1823: 'EVENT_IFACE', # 0x71f
            1824: 'EVENTS_DS_GSM_L1_ALIGN_VFR', # 0x720
            1825: 'EVENTS_DS_GSM_L1_STATE', # 0x721
            1826: 'EVENTS_DS_GSM_RATSCCH_IN_DTX', # 0x722
            1827: 'EVENTS_DS_GSM_FACCH_IN_DTX', # 0x723
            1828: 'EVENTS_DS_GSM_FACCH_AND_RATSCCH_COLLISION', # 0x724
            1829: 'EVENTS_DS_GSM_FACCH_AND_SID_UPDATE_COLLISION', # 0x725
            1830: 'EVENTS_DS_GSM_RATSCCH_AND_SID_UPDATE_COLLISION', # 0x726
            1831: 'EVENTS_DS_GSM_AMR_STATE_CHANGE', # 0x727
            1832: 'EVENTS_DS_GSM_RATSCCH_CMI_PHASE_CHANGE', # 0x728
            1833: 'EVENTS_DS_GSM_RATSCCH_REQ_ACT_TIMER_EXPIRY', # 0x729
            1834: 'EVENTS_DS_GSM_RATSCCH_ACK_ACT_TIMER_EXPIRY', # 0x72a
            1835: 'EVENTS_DS_GSM_AMR_RATSCCH_REQ', # 0x72b
            1836: 'EVENTS_DS_GSM_AMR_RATSCCH_RSP', # 0x72c
            1837: 'EVENTS_DS_GSM_AMR_CMC_TURNAROUND_TIME', # 0x72d
            1838: 'EVENTS_DS_GPRS_SMGMM_MSG_RECEIVED', # 0x72e
            1839: 'EVENTS_DS_GPRS_SMGMM_MSG_SENT', # 0x72f
            1840: 'EVENTS_DS_GPRS_LLC_READY_TIMER_START', # 0x730
            1841: 'EVENTS_DS_GPRS_LLC_READY_TIMER_END', # 0x731
            1842: 'EVENTS_DS_PACKET_TIMESLOT_RECONFIGURE', # 0x732
            1843: 'EVENTS_DS_GPRS_MAC_MSG_RECEIVED', # 0x733
            1844: 'EVENTS_DS_GPRS_MAC_MSG_SENT', # 0x734
            1845: 'EVENTS_DS_GPRS_MAC_CAMPED_ON_CELL', # 0x735
            1846: 'EVENTS_DS_GPRS_CELL_CHANGE_FAILURE', # 0x736
            1847: 'EVENTS_DS_GPRS_PACKET_CHANNEL_REQUEST', # 0x737
            1848: 'EVENTS_DS_GPRS_PACKET_UPLINK_ASSIGNMENT', # 0x738
            1849: 'EVENTS_DS_GPRS_PACKET_DOWNLINK_ASSIGNMENT', # 0x739
            1850: 'EVENTS_DS_GPRS_TBF_RELEASE', # 0x73a
            1851: 'EVENTS_DS_GPRS_TIMER_EXPIRY', # 0x73b
            1852: 'EVENTS_DS_GPRS_PACKET_RESOURCE_REQUEST', # 0x73c
            1853: 'EVENTS_DS_RANDOM_ACCESS_REQUEST', # 0x73d
            1854: 'EVENTS_DS_GSM_HANDOVER_START', # 0x73e
            1855: 'EVENTS_DS_GSM_HANDOVER_END', # 0x73f
            1856: 'EVENTS_DS_GSM_RESELECT_START', # 0x740
            1857: 'EVENTS_DS_GSM_RESELECT_END', # 0x741
            1858: 'EVENTS_DS_GSM_TO_WCDMA_RESELECT_END', # 0x742
            1859: 'EVENTS_DS_GSM_MESSAGE_RECEIVED', # 0x743
            1860: 'EVENTS_DS_GSM_RR_IN_SERVICE', # 0x744
            1861: 'EVENTS_DS_GSM_RR_OUT_OF_SERVICE', # 0x745
            1862: 'EVENTS_DS_GSM_PAGE_RECEIVED', # 0x746
            1863: 'EVENTS_DS_GSM_CAMP_ATTEMPT_START', # 0x747
            1864: 'EVENTS_DS_GSM_CAMP_ATTEMPT_END', # 0x748
            1865: 'EVENTS_DS_GSM_CALL_DROP', # 0x749
            1866: 'EVENTS_DS_GSM_ACCESS_FAILURE', # 0x74a
            1867: 'EVENTS_DS_GSM_CELL_SELECTION_START', # 0x74b
            1868: 'EVENTS_DS_GSM_CELL_SELECTION_END', # 0x74c
            1869: 'EVENTS_DS_GSM_POWER_SCAN_STATUS', # 0x74d
            1870: 'EVENTS_DS_GSM_PLMN_LIST_START', # 0x74e
            1871: 'EVENTS_DS_GSM_PLMN_LIST_END', # 0x74f
            1872: 'EVENTS_DS_GSM_AMR_MULTIRATE_IE', # 0x750
            1873: 'EVENTS_DS_GPRS_LINK_FAILURE', # 0x751
            1874: 'EVENTS_DS_GPRS_PAGE_RECEIVED', # 0x752
            1875: 'EVENTS_DS_GPRS_SURROUND_SEARCH_START', # 0x753
            1876: 'EVENTS_DS_GPRS_SURROUND_SEARCH_END', # 0x754
            1877: 'EVENTS_DS_GPRS_EARLY_CAMPING', # 0x755
            1878: 'EVENTS_DS_GSM_LINK_FAILURE', # 0x756
            1879: 'EVENT_MTP_FILE_DELETED', # 0x757
            1880: 'EVENT_MTP_PLAYLIST_REMOVED_OBJECT', # 0x758
            1881: 'EVENT_MTP_SYNC_STARTED', # 0x759
            1882: 'EVENT_MTP_SYNC_FINISHED', # 0x75a
            1883: 'EVENT_MTP_SAVE_ALBUMART_STARTED', # 0x75b
            1884: 'EVENT_MTP_SAVE_ALBUMART_FINISHED', # 0x75c
            1885: 'EVENT_MTP_FORMAT_STORE_STARTED', # 0x75d
            1886: 'EVENT_MTP_FORMAT_STORE_DONE', # 0x75e
            1887: 'EVENT_MTP_FORMAT_STORE_ERROR', # 0x75f
            1888: 'EVENT_LTE_RRC_SECURITY_CONFIG', # 0x760
            1889: 'EVENT_LTE_RRC_IRAT_RESEL_FROM_EUTRAN_START', # 0x761
            1890: 'EVENT_LTE_RRC_IRAT_RESEL_FROM_EUTRAN_END', # 0x762
            1891: 'EVENT_SNS_REST_DETECT_ACCEL_ACTIVE_TS', # 0x763
            1892: 'EVENT_SNS_REST_DETECT_ACCEL_STOP_TS', # 0x764
            1893: 'EVENT_CPC_CONFIG_ACTION', # 0x765
            1894: 'EVENT_FDPCH_CONFIG_ACTION', # 0x766
            1895: 'EVENT_SNS_DRV_MOTION_DETECT_SIG', # 0x767
            1896: 'EVENT_SNS_DRV_OPMODE_CHANGE', # 0x768
            2701: 'EVENT_WLAN_EAPOL', # 0xa8d
            2722: 'EVENT_WLAN_WAKE_LOCK', # 0xaa2
            2726: 'EVENT_WLAN_BEACON_RECEIVED', # 0xaa6
            2727: 'EVENT_WLAN_LOG_COMPLETE', # 0xaa7
            2739: 'EVENT_WLAN_STATUS_V2', # 0xab3
            2741: 'EVENT_WLAN_TDLS_TEARDOWN', # 0xab5
            2742: 'EVENT_WLAN_TDLS_ENABLE_LINK', # 0xab6
            2743: 'EVENT_WLAN_SUSPEND_RESUME', # 0xab7
            2744: 'EVENT_WLAN_OFFLOAD_REQ', # 0xab8
            2745: 'EVENT_TDLS_SCAN_BLOCK', # 0xab9
            2746: 'EVENT_WLAN_TDLS_TX_RX_MGMT', # 0xaba
            2747: 'EVENT_WLAN_LOW_RESOURCE_FAILURE', # 0xabb
        }

    def parse_event_fallback(self, ts, event_id, *args):
        gsmtap_hdr = util.create_gsmtap_header(
            version = 2,
            payload_type = util.gsmtap_type.OSMOCORE_LOG)
        osmocore_log_hdr = util.create_osmocore_logging_header(
            timestamp = ts,
            process_name = b'Event',
            pid = event_id,
        )
        if event_id in self.event_names:
            log_precontent = '{}: '.format(self.event_names[event_id]).encode('utf-8')
        else:
            log_precontent = 'Event {}: '.format(event_id).encode('utf-8')

        header = gsmtap_hdr + osmocore_log_hdr + log_precontent
        log_content = b''

        if len(args) == 2:
            # arg1, arg2 -> 2x uint8
            log_content = '{:#02x} {:#02x}'.format(args[0], args[1]).encode('utf-8')
        elif len(args) == 1:
            if type(args[0]) == int:
                log_content = '{:#02x}'.format(args[0]).encode('utf-8')
            elif type(args[0]) == bytes:
                log_content = '{}'.format(' '.join('{:02x}'.format(x) for x in args[0])).encode('utf-8')
            else:
                if self.parent:
                    self.parent.logger.log(logging.WARNING, 'Unexpected type of arguments ({}) passed to event'.format(type(args[0])))
        elif len(args) == 0:
            pass
        else:
            if self.parent:
                self.parent.logger.log(logging.WARNING, 'Unexpected number of arguments ({}) passed to event'.format(len(args)))

        return header + log_content